@@ -14,7 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import copy
 import json
 import logging
 import os
@@ -22,7 +21,6 @@ import threading
 import time
 import tempfile
 import shutil
-import yaml
 from distutils.version import LooseVersion
 
 try:
@@ -47,10 +45,11 @@ except ImportError:
     # for Ansible version 2.3.2 or lower
     pass
 
+from IM.ansible_utils import merge_recipes
 from IM.ansible_utils.ansible_launcher import AnsibleThread
 
-import IM.InfrastructureManager
 import IM.InfrastructureList
+from IM.LoggerMixin import LoggerMixin
 from IM.VirtualMachine import VirtualMachine
 from IM.SSH import AuthenticationException
 from IM.SSHRetry import SSHRetry
@@ -59,7 +58,7 @@ from IM.config import Config
 from radl.radl import system, contextualize_item
 
 
-class ConfManager(threading.Thread):
+class ConfManager(LoggerMixin, threading.Thread):
     """
     Class to manage the contextualization steps
     """
@@ -76,10 +75,9 @@ class ConfManager(threading.Thread):
         self.max_ctxt_time = max_ctxt_time
         self._stop_thread = False
         self.ansible_process = None
-        self.failed_step = []
         self.logger = logging.getLogger('ConfManager')
 
-    def check_running_pids(self, vms_configuring, failed_step):
+    def check_running_pids(self, vms_configuring):
         """
         Update the status of the configuration processes
         """
@@ -98,7 +96,6 @@ class ConfManager(threading.Thread):
                         if vm.configured:
                             self.log_info("Configuration process of VM %s success." % vm.im_id)
                         elif vm.configured is False:
-                            failed_step.append(step)
                             self.log_info("Configuration process of VM %s failed." % vm.im_id)
                         else:
                             self.log_warn("Configuration process of VM %s in unfinished state." % vm.im_id)
@@ -116,14 +113,13 @@ class ConfManager(threading.Thread):
                         if vm.configured:
                             self.log_info("Configuration process of master node successfully finished.")
                         elif vm.configured is False:
-                            failed_step.append(step)
                             self.log_info("Configuration process of master node failed.")
                         else:
                             self.log_warn("Configuration process of master node in unfinished state.")
                         # Force to save the data to store the log data
                         IM.InfrastructureList.InfrastructureList.save_data(self.inf.id)
 
-        return failed_step, res
+        return res
 
     def stop(self):
         self._stop_thread = True
@@ -134,10 +130,51 @@ class ConfManager(threading.Thread):
             self.log_info("Stopping pending Ansible process.")
             self.ansible_process.terminate()
 
-    def check_vm_ips(self, timeout=Config.WAIT_RUNNING_VM_TIMEOUT):
-
+    def wait_all_vm_ips(self, timeout=Config.ANSIBLE_INSTALL_TIMEOUT):
+        """
+        Assure that all the VMs of the Inf. have all the requested public IPs assigned
+        """
         wait = 0
-        # Assure that all the VMs of the Inf. have one IP
+        success = False
+        while not success and wait < timeout and not self._stop_thread:
+            success = True
+            for vm in self.inf.get_vm_list():
+
+                # If the VM is not in a "running" state, ignore it
+                if vm.state in VirtualMachine.NOT_RUNNING_STATES:
+                    self.log_warn("The VM ID: " + str(vm.id) +
+                                  " is not running, do not wait it to have an IP.")
+                    continue
+
+                if vm.hasPublicNet():
+                    self.log_debug("VM %s requests a public IP." % vm.id)
+                    if not vm.getPublicIP():
+                        self.log_debug("And it does not have it assigned yet.")
+                        success = False
+                        vm.update_status(self.auth)
+
+            if not success:
+                self.log_warn("Still waiting all the VMs to have all the requested IPs")
+                wait += Config.CONFMAMAGER_CHECK_STATE_INTERVAL
+                time.sleep(Config.CONFMAMAGER_CHECK_STATE_INTERVAL)
+
+        if not success:
+            self.log_warn("Error waiting all the VMs to have all the requested IPs")
+        else:
+            self.log_info("All the VMs have all the requested IPs")
+            # do a final update of all VMs
+            for vm in self.inf.get_vm_list():
+                if vm.state not in VirtualMachine.NOT_RUNNING_STATES:
+                    vm.update_status(self.auth)
+
+        return success
+
+    def check_vm_ips(self, timeout=Config.WAIT_RUNNING_VM_TIMEOUT):
+        """
+        Assure that all the VMs of the Inf. have at least one IP
+        """
+        wait = 0
+
         success = False
         while not success and wait < timeout and not self._stop_thread:
             success = True
@@ -196,14 +233,13 @@ class ConfManager(threading.Thread):
             self.log_info("Killing ctxt processes in VM: %s" % vm.id)
             try:
                 vm.kill_check_ctxt_process()
-            except:
+            except Exception:
                 self.log_exception("Error killing ctxt processes in VM: %s" % vm.id)
             vm.configured = None
 
     def run(self):
         self.log_info("Starting the ConfManager Thread")
 
-        self.failed_step = []
         last_step = None
         vms_configuring = {}
 
@@ -225,7 +261,7 @@ class ConfManager(threading.Thread):
                         vm.configured = False
                 return
 
-            self.failed_step, vms_configuring = self.check_running_pids(vms_configuring, self.failed_step)
+            vms_configuring = self.check_running_pids(vms_configuring)
 
             # If the queue is empty but there are vms configuring wait and test
             # again
@@ -242,13 +278,11 @@ class ConfManager(threading.Thread):
 
             # if this task is from a next step
             if last_step is not None and last_step < step:
-                if self.failed_step and sorted(self.failed_step)[-1] < step:
-                    self.log_info("Configuration of process of step %s failed, "
-                                  "ignoring tasks of step %s." % (sorted(self.failed_step)[-1], step))
-                    vm.configured = False
+                if vm.is_configured() is False:
+                    self.log_debug("Configuration process of step " + str(last_step) +
+                                   " failed, ignoring tasks of later steps.")
                 else:
-                    # Add the task again to the queue only if the last step was
-                    # OK
+                    # Add the task again to the queue only if the last step was OK
                     self.inf.add_ctxt_tasks([(step, prio, vm, tasks)])
 
                     # If there are any process running of last step, wait
@@ -256,8 +290,7 @@ class ConfManager(threading.Thread):
                         self.log_info("Waiting processes of step " + str(last_step) + " to finish.")
                         time.sleep(Config.CONFMAMAGER_CHECK_STATE_INTERVAL)
                     else:
-                        # if not, update the step, to go ahead with the new
-                        # step
+                        # if not, update the step, to go ahead with the new step
                         self.log_info("Step " + str(last_step) + " finished. Go to step: " + str(step))
                         last_step = step
             else:
@@ -333,8 +366,7 @@ class ConfManager(threading.Thread):
                 self.log_error("VM with ID %s (%s) does not have an IP!!. "
                                "We cannot launch the ansible process!!" % (str(vm.im_id), vm.id))
             else:
-                remote_dir = Config.REMOTE_CONF_DIR + "/" + \
-                    str(self.inf.id) + "/" + ip + "_" + str(vm.im_id)
+                remote_dir = Config.REMOTE_CONF_DIR + "/" + str(self.inf.id) + "/" + ip + "_" + str(vm.im_id)
                 tmp_dir = tempfile.mkdtemp()
 
                 self.log_info("Create the configuration file for the contextualization agent")
@@ -346,8 +378,7 @@ class ConfManager(threading.Thread):
                 # Copy the contextualization agent config file
                 ssh = vm.get_ssh_ansible_master()
                 ssh.sftp_mkdir(remote_dir)
-                ssh.sftp_put(conf_file, remote_dir + "/" +
-                             os.path.basename(conf_file))
+                ssh.sftp_put(conf_file, remote_dir + "/" + os.path.basename(conf_file))
 
                 if vm.configured is None:
                     if len(self.inf.get_vm_list()) > Config.VM_NUM_USE_CTXT_DIST:
@@ -373,7 +404,7 @@ class ConfManager(threading.Thread):
                     vm.launch_check_ctxt_process()
                 else:
                     self.log_warn("Ansible process to configure " + str(vm.im_id) + " NOT launched")
-        except:
+        except Exception:
             pid = None
             self.log_exception("Error launching the ansible process to configure VM with ID %s" % str(vm.im_id))
         finally:
@@ -412,14 +443,7 @@ class ConfManager(threading.Thread):
         vm_group = self.inf.get_vm_list_by_system_name()
         for group in vm_group:
             vm = vm_group[group][0]
-            user = vm.getCredentialValues()[0]
             out.write('[' + group + ':vars]\n')
-            if user:
-                out.write('ansible_user=' + user + '\n')
-                # For compatibility with Ansible 1.X versions
-                out.write('ansible_ssh_user=' + user + '\n')
-            else:
-                self.log_warn("The VM ID: " + str(vm.id) + " does not have username!!")
 
             if vm.getOS().lower() == "windows":
                 out.write('ansible_connection=winrm\n')
@@ -486,6 +510,14 @@ class ConfManager(threading.Thread):
                 # For compatibility with Ansible 1.X versions
                 node_line += ' ansible_ssh_port=%d' % vm.getRemoteAccessPort()
 
+                user = vm.getCredentialValues()[0]
+                if user:
+                    node_line += ' ansible_user=%s' % user
+                    # For compatibility with Ansible 1.X versions
+                    node_line += ' ansible_ssh_user=%s' % user
+                else:
+                    self.log_warn("The VM ID: " + str(vm.id) + " does not have username!!")
+
                 if self.inf.vm_master and vm.id == self.inf.vm_master.id:
                     node_line += ' ansible_connection=local'
 
@@ -607,7 +639,8 @@ class ConfManager(threading.Thread):
         recipe_files.append("basic_task_all.yml")
         return recipe_files
 
-    def generate_mount_disks_tasks(self, system):
+    @staticmethod
+    def generate_mount_disks_tasks(system):
         """
         Generate a set of tasks to format and mount the specified disks
         """
@@ -624,29 +657,18 @@ class ConfManager(threading.Thread):
                 if disk_mount_path and disk_fstype:
                     # This recipe works with EC2, OpenNebula and Azure. It must be
                     # tested/completed with other providers
-                    condition = "    when: ansible_os_family != 'Windows' and item.key.endswith('" + \
-                        disk_device[-1] + "')"
-                    with_dict = "    with_dict: '{{ ansible_devices }}'\n"
-
-                    res += '  # Tasks to format and mount disk %d from device %s in %s\n' % (cont,
-                                                                                             disk_device,
-                                                                                             disk_mount_path)
-                    res += '  - shell: (echo n; echo p; echo 1; echo ; echo; echo w) |'
-                    res += ' fdisk /dev/{{item.key}} creates=/dev/{{item.key}}1\n'
-                    # res += '  - parted: device=/dev/{{item.key}} number=1 state=present'
-                    res += condition + '\n'
-                    res += with_dict
-                    res += '  - filesystem: fstype=' + disk_fstype + ' dev=/dev/{{item.key}}1\n'
-                    res += '    ignore_errors: yes\n'
-                    res += '    register: format\n'
-                    res += condition + '\n'
-                    res += with_dict
-                    res += '  - file: path=' + disk_mount_path + ' state=directory recurse=yes\n'
-                    res += '  - mount: name=' + disk_mount_path + ' src=/dev/{{item.key}}1 state=mounted fstype=' + \
-                        disk_fstype + '\n'
-                    res += condition + ' and not format is failed\n'
-                    res += with_dict
-                    res += '\n'
+                    res += '  - include: utils/tasks/disk_format_mount.yml\n'
+                    res += '    vars:\n'
+                    res += '      device: "/dev/{{item.key}}"\n'
+                    res += '      mount_path: "' + disk_mount_path + '"\n'
+                    res += '      fstype: "' + disk_fstype + '"\n'
+                    res += "    with_dict: '{{ ansible_devices }}'\n"
+                    res += "    when: ansible_os_family != 'Windows' and ("
+                    # Devices hdb, sdb, xvdb, etc
+                    res += "item.key.endswith('d%s') or " % disk_device[-1]
+                    # Devices nvme0n1 (NVMe type in EC2)
+                    res += "item.key.startswith('nvme%sn1')" % (ord(disk_device[-1]) - 97)
+                    res += ")\n"
 
             cont += 1
 
@@ -680,7 +702,7 @@ class ConfManager(threading.Thread):
 
             # If there are a recipe, use it
             if recipe:
-                conf_content = self.mergeYAML(conf_content, recipe)
+                conf_content = merge_recipes(conf_content, recipe)
                 conf_content += "\n\n"
             else:
                 # use the app name as the package to install
@@ -697,7 +719,7 @@ class ConfManager(threading.Thread):
                 install_app += "    action: yum pkg=" + short_app_name + " state=installed\n"
                 install_app += "    when: \"ansible_os_family == 'RedHat'\"\n"
                 install_app += "    ignore_errors: yes\n"
-                conf_content = self.mergeYAML(conf_content, install_app)
+                conf_content = merge_recipes(conf_content, install_app)
 
         conf_out.write(conf_content)
         conf_out.close()
@@ -712,7 +734,8 @@ class ConfManager(threading.Thread):
 
         return recipe_files
 
-    def get_vault_editor(self, vault_password):
+    @staticmethod
+    def get_vault_editor(vault_password):
         """
         Get the correct VaultEditor object in different Ansible versions
         """
@@ -742,10 +765,10 @@ class ConfManager(threading.Thread):
                     recipes = vault_edit.vault.decrypt(configure.recipes.strip())
                 else:
                     recipes = configure.recipes
-                conf_content = self.mergeYAML(conf_content, recipes)
+                conf_content = merge_recipes(conf_content, recipes)
                 conf_content = vault_edit.vault.encrypt(conf_content)
             else:
-                conf_content = self.mergeYAML(conf_content, configure.recipes)
+                conf_content = merge_recipes(conf_content, configure.recipes)
 
             conf_out = open(conf_filename, 'w')
             conf_out.write(conf_content)
@@ -785,7 +808,11 @@ class ConfManager(threading.Thread):
                     if self.inf.radl.ansible_hosts:
                         configured_ok = True
                     else:
+                        if not self.inf.vm_master:
+                            raise Exception("No master VM found.")
                         ssh = self.inf.vm_master.get_ssh(retry=True)
+                        if not ssh:
+                            raise Exception("Master VM does not have IP.")
                         # Activate tty mode to avoid some problems with sudo in
                         # REL
                         ssh.tty = True
@@ -808,6 +835,7 @@ class ConfManager(threading.Thread):
                         remote_dir = Config.REMOTE_CONF_DIR + "/" + str(self.inf.id) + "/"
                         self.log_info("Copy the contextualization agent files")
                         files = []
+                        files.append((Config.IM_PATH + "/CtxtAgentBase.py", remote_dir + "/IM/CtxtAgentBase.py"))
                         files.append((Config.IM_PATH + "/SSH.py", remote_dir + "/IM/SSH.py"))
                         files.append((Config.IM_PATH + "/SSHRetry.py", remote_dir + "/IM/SSHRetry.py"))
                         files.append((Config.IM_PATH + "/retry.py", remote_dir + "/IM/retry.py"))
@@ -824,6 +852,7 @@ class ConfManager(threading.Thread):
                             for ansible_host in self.inf.radl.ansible_hosts:
                                 (user, passwd, private_key) = ansible_host.getCredentialValues()
                                 ssh = SSHRetry(ansible_host.getHost(), user, passwd, private_key)
+                                ssh.sftp_mkdir(Config.REMOTE_CONF_DIR, 0o755)
                                 ssh.sftp_mkdir(remote_dir, 0o700)
                                 ssh.sftp_mkdir(remote_dir + "/IM")
                                 ssh.sftp_put_files(files)
@@ -909,11 +938,6 @@ class ConfManager(threading.Thread):
                     self.inf.set_configured(False)
                     return
 
-                # To avoid problems with the known hosts of previous calls
-                if os.path.isfile(os.path.expanduser("~/.ssh/known_hosts")):
-                    self.log_debug("Remove " + os.path.expanduser("~/.ssh/known_hosts"))
-                    os.remove(os.path.expanduser("~/.ssh/known_hosts"))
-
                 self.inf.add_cont_msg("Wait master VM to have the SSH active.")
                 is_connected, msg = self.wait_vm_ssh_acccess(self.inf.vm_master, Config.WAIT_SSH_ACCCESS_TIMEOUT)
                 if not is_connected:
@@ -936,7 +960,7 @@ class ConfManager(threading.Thread):
                 IM.InfrastructureList.InfrastructureList.save_data(self.inf.id)
 
                 self.inf.set_configured(True)
-            except:
+            except Exception:
                 self.log_exception("Error waiting the master VM to be running")
                 self.inf.set_configured(False)
         else:
@@ -1086,20 +1110,24 @@ class ConfManager(threading.Thread):
                     self.log_warn('VM: ' + str(vm.id) + " is in state Failed. Does not wait for SSH.")
                     return False, "VM Failure."
 
+                ssh = None
                 ip = vm.getPublicIP()
-                if ip is not None:
+                if ip is not None or (vm.getPrivateIP() and vm.getProxyHost()):
                     ssh = vm.get_ssh()
-                    self.log_info('SSH Connecting with: ' + ip + ' to the VM: ' + str(vm.id))
+                    msg = ""
+                    if vm.getProxyHost():
+                        msg = " (via %s)" % vm.getProxyHost().host
+                    self.log_info('SSH Connecting with: ' + ssh.host + msg + ' to the VM: ' + str(vm.id))
 
                     try:
                         connected = ssh.test_connectivity(5)
                     except AuthenticationException:
-                        self.log_warn("Error connecting with ip: " + ip + " incorrect credentials.")
+                        self.log_warn("Error connecting with ip: " + ssh.host + " incorrect credentials.")
                         auth_errors += 1
 
                         if auth_errors >= auth_error_retries:
                             self.log_error("Too many authentication errors")
-                            return False, "Error connecting with ip: " + ip + " incorrect credentials."
+                            return False, "Error connecting with ip: " + ssh.host + " incorrect credentials."
 
                     if connected:
                         self.log_info('Works!')
@@ -1172,7 +1200,7 @@ class ConfManager(threading.Thread):
 
                 if change_creds:
                     self.inf.vm_master.info.systems[0].updateNewCredentialValues()
-        except:
+        except Exception:
             self.log_exception("Error changing credentials to master VM.")
 
         return change_creds
@@ -1222,7 +1250,7 @@ class ConfManager(threading.Thread):
                 try:
                     # Try to assure that the are no ansible process running
                     self.ansible_process.teminate()
-                except:
+                except Exception:
                     self.log_exception('Problems terminating Ansible processes.')
                 self.ansible_process = None
                 return (False, "Timeout. Ansible process terminated.")
@@ -1238,15 +1266,16 @@ class ConfManager(threading.Thread):
             _, return_code, output = result.get(timeout=10, block=False)
             msg = output.getvalue()
             self.log_info('Results obtained')
-        except:
+        except Exception:
             self.log_exception('Error getting ansible results.')
             return_code = 1
             msg = "Error getting ansible results."
 
         try:
             # Try to assure that the are no ansible process running
+            self.log_debug("Terminating ansible process: %s." % self.ansible_process.pid)
             self.ansible_process.teminate()
-        except:
+        except Exception:
             self.log_exception('Problems terminating Ansible processes.')
         self.ansible_process = None
 
@@ -1255,7 +1284,8 @@ class ConfManager(threading.Thread):
         else:
             return (False, msg)
 
-    def add_ansible_header(self, os_type, gather_facts=False):
+    @staticmethod
+    def add_ansible_header(os_type, gather_facts=False):
         """
         Add the IM needed header in the contextualization playbooks
 
@@ -1272,7 +1302,8 @@ class ConfManager(threading.Thread):
 
         return conf_content
 
-    def create_all_recipe(self, tmp_dir, filename, group="allnowindows", suffix="_all.yml"):
+    @staticmethod
+    def create_all_recipe(tmp_dir, filename, group="allnowindows", suffix="_all.yml"):
         """
         Create the recipe "all" enabling to access all the ansible variables from all hosts
         Arguments:
@@ -1300,8 +1331,35 @@ class ConfManager(threading.Thread):
         try:
             # Create the ansible inventory file
             with open(tmp_dir + "/inventory.cfg", 'w') as inv_out:
-                inv_out.write("%s  ansible_port=%d  ansible_ssh_port=%d" % (
-                    ssh.host, ssh.port, ssh.port))
+                ssh_args = ""
+                if ssh.proxy_host:
+                    # if user set the private key
+                    if ssh.proxy_host.private_key:
+                        priv_key_filename = "/var/tmp/%s_%s_%s.pem" % (ssh.proxy_host.username,
+                                                                       ssh.username,
+                                                                       ssh.host)
+                        # copy it to the proxy host to enable im_client to use it
+                        # ssh.proxy_host.sftp_put_content(ssh.proxy_host.private_key, priv_key_filename)
+                        # ssh.proxy_host.sftp_chmod(priv_key_filename, 0o600)
+                        # we must create it in the localhost
+                        with open(priv_key_filename, 'w') as f:
+                            f.write(ssh.proxy_host.private_key)
+                        os.chmod(priv_key_filename, 0o600)
+
+                        proxy_command = "ssh -W %%h:%%p -i %s -p %d %s %s@%s" % (priv_key_filename,
+                                                                                 ssh.proxy_host.port,
+                                                                                 "-o StrictHostKeyChecking=no",
+                                                                                 ssh.proxy_host.username,
+                                                                                 ssh.proxy_host.host)
+                    else:
+                        proxy_command = "sshpass -p %s ssh -W %%h:%%p -p %d %s %s@%s" % (ssh.proxy_host.password,
+                                                                                         ssh.proxy_host.port,
+                                                                                         "-o StrictHostKeyChecking=no",
+                                                                                         ssh.proxy_host.username,
+                                                                                         ssh.proxy_host.host)
+                    ssh_args = "ansible_ssh_extra_args=\" -oProxyCommand='%s'\"" % proxy_command
+                inv_out.write("%s  ansible_port=%d  ansible_ssh_port=%d %s" % (ssh.host, ssh.port,
+                                                                               ssh.port, ssh_args))
 
             shutil.copy(Config.CONTEXTUALIZATION_DIR + "/" +
                         ConfManager.MASTER_YAML, tmp_dir + "/" + ConfManager.MASTER_YAML)
@@ -1347,13 +1405,12 @@ class ConfManager(threading.Thread):
                     cmd = "echo '" + ssh.password + "' | " + cmd
                 (stdout, stderr, _) = ssh.execute(cmd, 120)
                 self.log_info(stdout + "\n" + stderr)
-            except:
+            except Exception:
                 self.log_exception("Error removing requiretty. Ignoring.")
 
             self.inf.add_cont_msg("Configure Ansible in the master VM.")
             self.log_info("Call Ansible to (re)configure in the master node")
-            (success, msg) = self.call_ansible(
-                tmp_dir, "inventory.cfg", ConfManager.MASTER_YAML, ssh)
+            (success, msg) = self.call_ansible(tmp_dir, "inventory.cfg", ConfManager.MASTER_YAML, ssh)
 
             if not success:
                 self.log_error("Error configuring master node: " + msg + "\n\n")
@@ -1378,7 +1435,10 @@ class ConfManager(threading.Thread):
             for req_app in s.getApplications():
                 if req_app.getValue("name").startswith("ansible.modules."):
                     # Get the modules specified by the user in the RADL
-                    modules.append(req_app.getValue("name")[16:])
+                    app_name = req_app.getValue("name")[16:]
+                    if req_app.getValue("version"):
+                        app_name += ",%s" % req_app.getValue("version")
+                    modules.append(app_name)
                 else:
                     # Get the info about the apps from the recipes DB
                     vm_modules, _ = Recipe.getInfoApps([req_app])
@@ -1426,8 +1486,8 @@ class ConfManager(threading.Thread):
                          'new_public_key'], vm_conf_data['new_private_key']) = new_creds
 
                 if not vm_conf_data['ip']:
-                    # if the vm does not have an IP, do not iclude it to avoid
-                    # errors configurin gother VMs
+                    # if the vm does not have an IP, do not include it to avoid
+                    # errors configuring other VMs
                     self.log_warn("The VM ID: " + str(vm.id) +
                                   " does not have an IP, do not include in the general conf file.")
                     self.inf.add_cont_msg("WARNING: The VM ID: " + str(vm.id) +
@@ -1435,8 +1495,9 @@ class ConfManager(threading.Thread):
                 else:
                     conf_data['vms'].append(vm_conf_data)
 
-        conf_data['conf_dir'] = Config.REMOTE_CONF_DIR + \
-            "/" + str(self.inf.id) + "/"
+                vm_conf_data['nat_instance'] = vm.info.systems[0].getValue('nat_instance') == 'yes'
+
+        conf_data['conf_dir'] = Config.REMOTE_CONF_DIR + "/" + str(self.inf.id) + "/"
 
         conf_out = open(conf_file, 'w')
         json.dump(conf_data, conf_out, indent=2)
@@ -1464,76 +1525,3 @@ class ConfManager(threading.Thread):
         self.log_debug("Ctxt agent vm configuration file: " + json.dumps(conf_data))
         json.dump(conf_data, conf_out, indent=2)
         conf_out.close()
-
-    def mergeYAML(self, yaml1, yaml2):
-        """
-        Merge two ansible yaml docs
-
-        Arguments:
-           - yaml1(str): string with the first YAML
-           - yaml1(str): string with the second YAML
-        Returns: The merged YAML. In case of errors, it concatenates both strings
-        """
-        yamlo1o = {}
-        try:
-            yamlo1o = yaml.safe_load(yaml1)[0]
-            if not isinstance(yamlo1o, dict):
-                yamlo1o = {}
-        except Exception:
-            self.log_exception("Error parsing YAML: " + yaml1 + "\n Ignore it")
-
-        try:
-            yamlo2s = yaml.safe_load(yaml2)
-            if not isinstance(yamlo2s, list) or any([not isinstance(d, dict) for d in yamlo2s]):
-                yamlo2s = {}
-        except Exception:
-            self.log_exception("Error parsing YAML: " + yaml2 + "\n Ignore it")
-            yamlo2s = {}
-
-        if not yamlo2s and not yamlo1o:
-            return ""
-
-        result = []
-        for yamlo2 in yamlo2s:
-            yamlo1 = copy.deepcopy(yamlo1o)
-            all_keys = []
-            all_keys.extend(yamlo1.keys())
-            all_keys.extend(yamlo2.keys())
-            all_keys = set(all_keys)
-
-            for key in all_keys:
-                if key in yamlo1 and yamlo1[key]:
-                    if key in yamlo2 and yamlo2[key]:
-                        if isinstance(yamlo1[key], dict):
-                            yamlo1[key].update(yamlo2[key])
-                        elif isinstance(yamlo1[key], list):
-                            yamlo1[key].extend(yamlo2[key])
-                        else:
-                            # Both use have the same key with merge in a lists
-                            v1 = yamlo1[key]
-                            v2 = yamlo2[key]
-                            yamlo1[key] = [v1, v2]
-                elif key in yamlo2 and yamlo2[key]:
-                    yamlo1[key] = yamlo2[key]
-            result.append(yamlo1)
-
-        return yaml.safe_dump(result, default_flow_style=False, explicit_start=True, width=256)
-
-    def log_msg(self, level, msg, exc_info=0):
-        msg = "Inf ID: %s: %s" % (self.inf.id, msg)
-        self.logger.log(level, msg, exc_info=exc_info)
-
-    def log_error(self, msg):
-        self.log_msg(logging.ERROR, msg)
-
-    def log_debug(self, msg):
-        self.log_msg(logging.DEBUG, msg)
-
-    def log_warn(self, msg):
-        self.log_msg(logging.WARNING, msg)
-
-    def log_exception(self, msg):
-        self.log_msg(logging.ERROR, msg, exc_info=1)
-
-    def log_info(self, msg):
-        self.log_msg(logging.INFO, msg)
