@@ -55,6 +55,14 @@ class DeletedVMException(Exception):
         self.message = msg
 
 
+class IncorrectStateException(Exception):
+    """ Invalid State. """
+
+    def __init__(self, msg="Invalid State to perform this operation."):
+        Exception.__init__(self, msg)
+        self.message = msg
+
+
 class InfrastructureInfo:
     """
     Stores all the information about a registered infrastructure.
@@ -105,6 +113,10 @@ class InfrastructureInfo:
         """ Time of the last access to this Inf. """
         self.snapshots = []
         """ List of URLs of snapshots made to this Inf that must be deleted on finalization """
+        self.adding = False
+        """Flag to specify that this Inf is adding resources """
+        self.deleting = False
+        """Flag to specify that this Inf is deleting resources """
 
     def serialize(self):
         with self._lock:
@@ -114,6 +126,8 @@ class InfrastructureInfo:
         del odict['_lock']
         del odict['ctxt_tasks']
         del odict['conf_threads']
+        del odict['adding']
+        del odict['deleting']
         if 'last_access' in odict:
             del odict['last_access']
         if odict['vm_master']:
@@ -142,10 +156,12 @@ class InfrastructureInfo:
             dic['auth'] = Authentication.deserialize(dic['auth'])
         if dic['radl']:
             dic['radl'] = parse_radl(dic['radl'])
+        else:
+            dic['radl'] = RADL()
         if 'extra_info' in dic and dic['extra_info'] and "TOSCA" in dic['extra_info']:
             try:
                 dic['extra_info']['TOSCA'] = Tosca.deserialize(dic['extra_info']['TOSCA'])
-            except:
+            except Exception:
                 del dic['extra_info']['TOSCA']
                 InfrastructureInfo.logger.exception("Error deserializing TOSCA document")
         newinf.__dict__.update(dic)
@@ -160,6 +176,8 @@ class InfrastructureInfo:
             if vm.im_id == vm_master_id:
                 newinf.vm_master = vm
             newinf.vm_list.append(vm)
+        newinf.adding = False
+        newinf.deleting = False
         return newinf
 
     @staticmethod
@@ -175,12 +193,11 @@ class InfrastructureInfo:
             newinf.auth = Authentication.deserialize(dic['auth'])
         return newinf
 
-    def destroy(self, auth, delete_list=None):
+    def destroy_vms(self, auth):
         """
-        Destroy the VMs listed in delete_list or all if not specified
+        Destroy all the VMs
         """
-        if delete_list is None:
-            delete_list = list(reversed(self.get_vm_list()))
+        delete_list = list(reversed(self.get_vm_list()))
 
         exceptions = []
         if Config.MAX_SIMULTANEOUS_LAUNCHES > 1:
@@ -224,6 +241,27 @@ class InfrastructureInfo:
         with self._lock:
             self.ctxt_tasks = PriorityQueue()
 
+    def destroy(self, auth, force=False):
+        """
+        Destroy the infrastructure
+        """
+        InfrastructureInfo.logger.info("Destroying the Inf ID: " + str(self.id))
+        try:
+            # First stop ctxt processes
+            self.stop()
+            # Destroy the Infrastructure
+            self.destroy_vms(auth)
+        except Exception as ex:
+            if not force:
+                raise ex
+        finally:
+            self.set_deleting(False)
+        # Set the Infrastructure as deleted
+        self.delete()
+        InfrastructureInfo.logger.info("Inf ID: %s: Successfully destroyed" % self.id)
+        IM.InfrastructureList.InfrastructureList.save_data(self.id)
+        IM.InfrastructureList.InfrastructureList.remove_inf(self)
+
     def get_cont_out(self):
         """
         Returns the contextualization message
@@ -251,7 +289,7 @@ class InfrastructureInfo:
         """
         try:
             str_msg = str(msg.decode('utf8', 'ignore'))
-        except:
+        except Exception:
             str_msg = msg
         self.cont_out += str(datetime.now()) + ": " + str_msg + "\n"
 
@@ -276,7 +314,7 @@ class InfrastructureInfo:
         """
         try:
             vm_id = int(str_vm_id)
-        except:
+        except Exception:
             raise IncorrectVMException()
 
         with self._lock:
@@ -300,7 +338,7 @@ class InfrastructureInfo:
                 groups[vm.getRequestedSystem().name] = [vm]
         return groups
 
-    def update_radl(self, radl, deployed_vms):
+    def update_radl(self, radl, deployed_vms, warn=True):
         """
         Update the stored radl with the passed one.
 
@@ -310,19 +348,21 @@ class InfrastructureInfo:
         - deployed_vms(list of tuple of deploy, system and list of VirtualMachines): list of
            tuples composed of the deploy, the concrete system deployed and the list of
            virtual machines deployed.
+        - warn(bool): Log a Warn message in case of redefinition
         """
 
         with self._lock:
             original_radl = self.radl.clone()
             # Add new networks ad ansible_hosts only
             for s in radl.networks + radl.ansible_hosts:
-                if not self.radl.add(s.clone(), "ignore"):
+                if not self.radl.add(s.clone(), "ignore") and warn:
                     InfrastructureInfo.logger.warn("Ignoring the redefinition of %s %s" % (type(s), s.getId()))
 
             # Add or update configures and systems
             for s in radl.configures + radl.systems:
+                if self.radl.get(s) and warn:
+                    InfrastructureInfo.logger.warn("(Re)definition of %s %s" % (type(s), s.getId()))
                 self.radl.add(s.clone(), "replace")
-                InfrastructureInfo.logger.warn("(Re)definition of %s %s" % (type(s), s.getId()))
 
             # Append contextualize
             self.radl.add(radl.contextualize)
@@ -427,15 +467,19 @@ class InfrastructureInfo:
         It will select the first created VM that fulfills this requirements
         and store the value in the vm_master field
         """
+        # If it was previously selected do no select a new one
+        if self.vm_master and not self.vm_master.destroy and not self.vm_master.deleting:
+            return self.vm_master
+
         self.vm_master = None
         max_vms_connected = -1
         for vm in self.get_vm_list():
             vms_connected = -1
-            if vm.getOS() and vm.getOS().lower() == 'linux' and vm.hasPublicNet():
+            if vm.getOS() and vm.getOS().lower() == 'linux' and (vm.hasPublicNet() or vm.getProxyHost()):
                 # check that is connected with other VMs
                 vms_connected = 0
                 for other_vm in self.get_vm_list():
-                    if not vm.isConnectedWith(other_vm):
+                    if vm.isConnectedWith(other_vm):
                         vms_connected += 1
 
                 if vms_connected > max_vms_connected:
@@ -545,10 +589,11 @@ class InfrastructureInfo:
                 max_ctxt_time = Config.MAX_CONTEXTUALIZATION_TIME
 
             ctxt_task = []
-            ctxt_task.append((-4, 0, self, ['kill_ctxt_processes']))
-            ctxt_task.append((-3, 0, self, ['check_vm_ips']))
-            ctxt_task.append((-2, 0, self, ['wait_master']))
-            ctxt_task.append((-1, 0, self, ['configure_master', 'generate_playbooks_and_hosts']))
+            ctxt_task.append((-5, 0, self, ['kill_ctxt_processes']))
+            ctxt_task.append((-4, 0, self, ['check_vm_ips']))
+            ctxt_task.append((-3, 0, self, ['wait_master']))
+            ctxt_task.append((-2, 0, self, ['configure_master', 'wait_all_vm_ips']))
+            ctxt_task.append((-1, 0, self, ['generate_playbooks_and_hosts']))
 
             use_dist = len(self.get_vm_list()) > Config.VM_NUM_USE_CTXT_DIST
             for cont, vm in enumerate(self.get_vm_list()):
@@ -640,6 +685,9 @@ class InfrastructureInfo:
                 if self_im_auth['password'] != password:
                     return False
 
+                # In case of OIDC token update it in each call to get a fresh version
+                self_im_auth['token'] = other_im_auth['token']
+
             if (self_im_auth['username'].startswith(InfrastructureInfo.OPENID_USER_PREFIX) and
                     'token' not in other_im_auth):
                 # This is a OpenID user do not enable to get data using user/pass creds
@@ -665,3 +713,22 @@ class InfrastructureInfo:
             return (datetime.now() - self.last_access > delay)
         else:
             return False
+
+    def set_deleting(self, value=True):
+        """
+        Set this inf as deleting
+        """
+        with self._lock:
+            if self.adding:
+                raise IncorrectStateException()
+            self.deleting = value
+
+    def set_adding(self, value=True):
+        """
+        Set this inf as adding
+        """
+        with self._lock:
+            if self.deleting:
+                self.add_cont_msg("Infrastructure deleted. Do not add resources.")
+                raise Exception("Infrastructure deleted. Do not add resources.")
+            self.adding = value
