@@ -14,6 +14,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import re
+import yaml
 import json
 import os
 import string
@@ -35,8 +37,6 @@ from radl import radl_parse
 from radl.radl import Feature, RADL
 from radl.radl_json import dump_radl as dump_radl_json
 
-from IM.config import Config
-from IM.VirtualMachine import VirtualMachine
 from IM.openid.JWT import JWT
 from IM.openid.OpenIDClient import OpenIDClient
 
@@ -90,6 +90,14 @@ class IncorrectVMCrecentialsException(Exception):
         self.message = msg
 
 
+class DisabledFunctionException(Exception):
+    """ Disabled function called"""
+
+    def __init__(self, msg="Function currently disabled."):
+        Exception.__init__(self, msg)
+        self.message = msg
+
+
 class InfrastructureManager:
     """
     Front-end to the functionality of the service.
@@ -172,9 +180,10 @@ class InfrastructureManager:
             requested_radl = radl.clone()
             requested_radl.systems = [radl.get_system_by_name(deploy.id)]
             if not concrete_system:
-                InfrastructureManager.logger.error(
-                    "Error, no concrete system to deploy: " + deploy.id + " in cloud: " +
-                    cloud_id + ". Check if a correct image is being used")
+                InfrastructureManager.logger.error("Inf ID: " + str(sel_inf.id) +
+                                                   ". Error, no concrete system to deploy: " +
+                                                   deploy.id + " in cloud: " + cloud_id +
+                                                   ". Check if a correct image is being used")
                 for _ in range(deploy.vm_number):
                     launched_vms.append((False, "Error, no concrete system to deploy: " + deploy.id +
                                          " in cloud: " + cloud_id + ". Check if a correct image is being used"))
@@ -189,8 +198,8 @@ class InfrastructureManager:
                     for _ in range(deploy.vm_number):
                         launched_vms.append((False, "No username for deploy: " + deploy.id))
                 else:
-                    InfrastructureManager.logger.debug(
-                        "Launching %d VMs of type %s" % (deploy.vm_number, concrete_system.name))
+                    InfrastructureManager.logger.debug("Inf ID: %s. Launching %d VMs of type %s" %
+                                                       (sel_inf.id, deploy.vm_number, concrete_system.name))
                     launched_vms = cloud.cloud.getCloudConnector(sel_inf).launch_with_retry(
                         sel_inf, launch_radl, requested_radl, deploy.vm_number, auth, Config.MAX_VM_FAILS,
                         Config.DELAY_BETWEEN_VM_RETRIES)
@@ -202,11 +211,13 @@ class InfrastructureManager:
 
             for success, launched_vm in launched_vms:
                 if success:
-                    InfrastructureManager.logger.debug("VM successfully launched: %s" % launched_vm.id)
+                    InfrastructureManager.logger.debug("Inf ID: %s. VM successfully launched: %s" % (sel_inf.id,
+                                                                                                     launched_vm.id))
                     deployed_vm.setdefault(deploy, []).append(launched_vm)
                     deploy.cloud_id = cloud_id
                 else:
-                    InfrastructureManager.logger.error("Error launching some of the VMs: %s" % launched_vm)
+                    InfrastructureManager.logger.error("Inf ID: %s. Error launching some of the "
+                                                       "VMs: %s" % (sel_inf.id, launched_vm))
                     vm = VirtualMachine(sel_inf, None, cloud.cloud, launch_radl, requested_radl)
                     vm.state = VirtualMachine.FAILED
                     vm.info.systems[0].setValue('state', VirtualMachine.FAILED)
@@ -224,6 +235,9 @@ class InfrastructureManager:
             InfrastructureManager.logger.error("Error, incorrect Inf ID: %s" % inf_id)
             raise IncorrectInfrastructureException()
         sel_inf = IM.InfrastructureList.InfrastructureList.get_infrastructure(inf_id)
+        if not sel_inf:
+            InfrastructureManager.logger.error("Error loading Inf ID: %s" % inf_id)
+            raise IncorrectInfrastructureException("Error loading Inf ID data.")
         if not sel_inf.is_authorized(auth):
             InfrastructureManager.logger.error("Access Error to Inf ID: %s" % inf_id)
             raise UnauthorizedUserException()
@@ -253,6 +267,9 @@ class InfrastructureManager:
 
         Return: "" if success.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Reconfiguring the Inf ID: " + str(inf_id))
@@ -267,6 +284,11 @@ class InfrastructureManager:
         # Update infrastructure RADL with this new RADL
         # Add or update configures
         for s in radl.configures:
+            # first check that the YAML is correct
+            try:
+                yaml.safe_load(s.recipes)
+            except Exception as ex:
+                raise Exception("Error parsing YAML: %s" % str(ex))
             sel_inf.radl.add(s.clone(), "replace")
             InfrastructureManager.logger.info(
                 "Inf ID: " + sel_inf.id + ": " +
@@ -289,6 +311,30 @@ class InfrastructureManager:
                         system.setCredentialValues(
                             password=password, public_key=public_key, private_key=private_key, new=True)
 
+                # The user has new applications
+                curr_apps = system.getValue("disk.0.applications")
+                curr_apps_names = {}
+                if curr_apps:
+                    for app_name in curr_apps.keys():
+                        orig_app_name = app_name
+                        if "," in app_name:
+                            # remove version substring
+                            pos = app_name.find(",")
+                            app_name = app_name[:pos]
+                        curr_apps_names[app_name] = orig_app_name
+
+                new_apps = new_system.getValue("disk.0.applications")
+                if new_apps:
+                    for app_name, app in new_apps.items():
+                        orig_app_name = app_name
+                        if "," in app_name:
+                            # remove version substring
+                            pos = app_name.find(",")
+                            app_name = app_name[:pos]
+                        if app_name in list(curr_apps_names.keys()):
+                            del curr_apps[curr_apps_names[app_name]]
+                        curr_apps[orig_app_name] = app
+
         # Stick all virtual machines to be reconfigured
         InfrastructureManager.logger.info("Contextualize the Inf ID: " + sel_inf.id)
         # reset ansible_configured to force the re-installation of galaxy roles
@@ -458,6 +504,9 @@ class InfrastructureManager:
 
         Return(list of int): ids of the new virtual machine created.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Adding resources to Inf ID: " + str(inf_id))
@@ -475,12 +524,14 @@ class InfrastructureManager:
 
             # Update infrastructure RADL with this new RADL
             sel_inf.complete_radl(radl)
+            sel_inf.update_radl(radl, [])
 
             # If any deploy is defined, only update definitions.
             if not radl.deploys:
-                sel_inf.update_radl(radl, [])
                 InfrastructureManager.logger.warn("Inf ID: " + sel_inf.id + ": without any deploy. Exiting.")
                 sel_inf.add_cont_msg("Infrastructure without any deploy. Exiting.")
+                if sel_inf.configured is None:
+                    sel_inf.configured = False
                 return []
         except Exception as ex:
             sel_inf.configured = False
@@ -507,7 +558,13 @@ class InfrastructureManager:
                         break
 
         # Concrete systems using VMRC
-        systems_with_vmrc = InfrastructureManager.systems_with_vmrc(sel_inf, radl, auth)
+        try:
+            systems_with_vmrc = InfrastructureManager.systems_with_vmrc(sel_inf, radl, auth)
+        except Exception as ex:
+            sel_inf.configured = False
+            sel_inf.add_cont_msg("Error getting VM images: %s" % str(ex))
+            InfrastructureManager.logger.exception("Inf ID: " + sel_inf.id + " error getting VM images")
+            raise ex
 
         # Concrete systems with cloud providers and select systems with the greatest score
         # in every cloud
@@ -533,12 +590,17 @@ class InfrastructureManager:
         deploys_group_cloud = InfrastructureManager.sort_by_score(sel_inf, concrete_systems, cloud_list,
                                                                   deploy_groups, auth)
 
+        # We are going to start adding resources
+        sel_inf.set_adding()
+
         # Launch every group in the same cloud provider
         deployed_vm = {}
         for deploy_group in deploy_groups:
             if not deploy_group:
                 InfrastructureManager.logger.warning("Inf ID: %s: No VMs to deploy!" % sel_inf.id)
                 sel_inf.add_cont_msg("No VMs to deploy. Exiting.")
+                if sel_inf.configured is None:
+                    sel_inf.configured = False
                 return []
 
             cloud_id = deploys_group_cloud[id(deploy_group)]
@@ -587,20 +649,40 @@ class InfrastructureManager:
                     random_password = ''.join(random.choice(string.ascii_letters + string.digits) for _ in range(8))
                     vm.info.systems[0].setCredentialValues(password=random_password, new=True)
 
+        error_msg = ""
         # Add the new virtual machines to the infrastructure
-        sel_inf.update_radl(radl, [(d, deployed_vm[d], concrete_systems[d.cloud_id][d.id][0]) for d in deployed_vm])
+        sel_inf.update_radl(radl,
+                            [(d, deployed_vm[d], concrete_systems[d.cloud_id][d.id][0]) for d in deployed_vm],
+                            False)
         if all_failed:
             InfrastructureManager.logger.error("VMs failed when adding to Inf ID: %s" % sel_inf.id)
             sel_inf.add_cont_msg("All VMs failed. No contextualize.")
+
+            # in case of all VMs are failed delete it
+            delete_list = list(reversed(sel_inf.get_vm_list()))
+            for vm in new_vms:
+                if vm.error_msg:
+                    error_msg += "%s\n" % vm.error_msg
+                vm.delete(delete_list, auth, [])
+            sel_inf.add_cont_msg(error_msg)
         else:
             InfrastructureManager.logger.info("VMs %s successfully added to Inf ID: %s" % (new_vms, sel_inf.id))
 
+        # The resources has been added
+        sel_inf.set_adding(False)
+
         # Let's contextualize!
         if context and new_vms and not all_failed:
             sel_inf.Contextualize(auth)
 
         IM.InfrastructureList.InfrastructureList.save_data(inf_id)
 
+        if all_failed and new_vms:
+            # if there are no VMs, set it as unconfigured
+            if not sel_inf.get_vm_list():
+                sel_inf.configured = False
+            raise Exception("Error adding VMs: %s" % error_msg)
+
         return [vm.im_id for vm in new_vms]
 
     @staticmethod
@@ -617,6 +699,9 @@ class InfrastructureManager:
 
         Return(int): number of undeployed virtual machines.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Removing the VMs: " + str(vm_list) + " from Inf ID: '" + str(inf_id) + "'")
@@ -748,6 +833,9 @@ class InfrastructureManager:
 
         Return: a str with the information about the VM
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info(
@@ -764,18 +852,10 @@ class InfrastructureManager:
         else:
             radl = radl_parse.parse_radl(radl_data)
 
-        exception = None
-        try:
-            (success, alter_res) = vm.alter(radl, auth)
-        except Exception as e:
-            exception = e
+        (success, alter_res) = vm.alter(radl, auth)
 
-        if exception:
-            raise exception
         if not success:
-            InfrastructureManager.logger.warn(
-                "Inf ID: " + str(inf_id) + ": " +
-                "Error modifying the information about the VM " + str(vm_id) + ": " + str(alter_res))
+            raise Exception("Error modifying the information about the VM %s: %s" % (vm_id, alter_res))
 
         vm.update_status(auth)
         IM.InfrastructureList.InfrastructureList.save_data(inf_id)
@@ -876,14 +956,15 @@ class InfrastructureManager:
 
         sel_inf = InfrastructureManager.get_infrastructure(inf_id, auth)
 
+        vm_list = sel_inf.get_vm_list()
         vm_states = {}
-        for vm in sel_inf.get_vm_list():
+        for vm in vm_list:
             # First try to update the status of the VM
             vm.update_status(auth)
             vm_states[str(vm.im_id)] = vm.state
 
         state = None
-        for vm in sel_inf.get_vm_list():
+        for vm in vm_list:
             # First try to update the status of the VM
             if vm.state == VirtualMachine.FAILED:
                 state = VirtualMachine.FAILED
@@ -912,9 +993,15 @@ class InfrastructureManager:
         if state is None:
             if sel_inf.configured is False:
                 state = VirtualMachine.FAILED
+            elif not vm_list and sel_inf.configured is None:
+                # if there are no vms we probably are in the vm creation process
+                state = VirtualMachine.PENDING
             else:
                 state = VirtualMachine.UNKNOWN
 
+        if sel_inf.deleting:
+            state = VirtualMachine.DELETING
+
         InfrastructureManager.logger.info("Inf ID: " + str(inf_id) + " is in state: " + state)
         return {'state': state, 'vm_states': vm_states}
 
@@ -942,6 +1029,9 @@ class InfrastructureManager:
 
         Return(str): error messages; empty string means all was ok.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Stopping the Inf ID: " + str(inf_id))
@@ -993,6 +1083,9 @@ class InfrastructureManager:
 
         Return(str): error messages; empty string means all was ok.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Starting the Inf ID: " + str(inf_id))
@@ -1033,6 +1126,9 @@ class InfrastructureManager:
 
         Return(str): error messages; empty string means all was ok.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
 
         InfrastructureManager.logger.info("Starting the VM id %s from the Inf ID: %s" % (vm_id, inf_id))
@@ -1068,6 +1164,9 @@ class InfrastructureManager:
 
         Return(str): error messages; empty string means all was ok.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         # First check the auth data
         auth = InfrastructureManager.check_auth_data(auth)
 
@@ -1093,7 +1192,47 @@ class InfrastructureManager:
             return ""
 
     @staticmethod
-    def DestroyInfrastructure(inf_id, auth):
+    def RebootVM(inf_id, vm_id, auth):
+        """
+        Reboot the specified virtual machine in an infrastructure
+
+        Args:
+
+        - inf_id(str): infrastructure id.
+        - vm_id(str): virtual machine id.
+        - auth(Authentication): parsed authentication tokens.
+
+        Return(str): error messages; empty string means all was ok.
+        """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
+        # First check the auth data
+        auth = InfrastructureManager.check_auth_data(auth)
+
+        InfrastructureManager.logger.info(
+            "Rebooting the VM id %s from the Inf ID: %s" % (vm_id, inf_id))
+
+        vm = InfrastructureManager.get_vm_from_inf(inf_id, vm_id, auth)
+        success = False
+        try:
+            (success, msg) = vm.reboot(auth)
+        except Exception as e:
+            msg = str(e)
+
+        if not success:
+            InfrastructureManager.logger.info(
+                "Inf ID: " + str(inf_id) + ": " +
+                "The VM %s cannot be rebooted: %s" % (vm_id, msg))
+            raise Exception("Error rebooting the VM: %s" % msg)
+        else:
+            InfrastructureManager.logger.info(
+                "Inf ID: " + str(inf_id) + ": " +
+                "The VM %s successfully rebooted" % vm_id)
+            return ""
+
+    @staticmethod
+    def DestroyInfrastructure(inf_id, auth, force=False, async_call=False):
         """
         Destroy all virtual machines in an infrastructure.
 
@@ -1101,25 +1240,29 @@ class InfrastructureManager:
 
         - inf_id(str): infrastructure id.
         - auth(Authentication): parsed authentication tokens.
+        - force(bool): delete the infra from the IM although not all resources are deleted.
+        - async_call(bool): Destroy the inf in an async way.
 
         Return: None.
         """
+        if Config.BOOT_MODE == 1:
+            raise DisabledFunctionException()
+
         # First check the auth data
         auth = InfrastructureManager.check_auth_data(auth)
 
-        InfrastructureManager.logger.info("Destroying the Inf ID: " + str(inf_id))
-
         sel_inf = InfrastructureManager.get_infrastructure(inf_id, auth)
-        delete_list = list(reversed(sel_inf.get_vm_list()))
-        # First stop ctxt processes
-        sel_inf.stop()
-        # Destroy the Infrastructure
-        sel_inf.destroy(auth, delete_list)
-        # Set the Infrastructure as deleted
-        sel_inf.delete()
-        IM.InfrastructureList.InfrastructureList.save_data(inf_id)
-        IM.InfrastructureList.InfrastructureList.remove_inf(sel_inf)
-        InfrastructureManager.logger.info("Inf ID: %s: Successfully destroyed" % inf_id)
+        # First set this infra as "deleting"
+        sel_inf.set_deleting()
+
+        if async_call:
+            t = threading.Thread(name="DestroyResource-%s" % sel_inf.id,
+                                 target=sel_inf.destroy,
+                                 args=(auth, force))
+            t.daemon = True
+            t.start()
+        else:
+            sel_inf.destroy(auth, force)
         return ""
 
     @staticmethod
@@ -1237,6 +1380,9 @@ class InfrastructureManager:
         if not im_auth:
             raise IncorrectVMCrecentialsException("No credentials provided for the InfrastructureManager.")
 
+        if Config.FORCE_OIDC_AUTH and "token" not in im_auth[0]:
+            raise IncorrectVMCrecentialsException("No token provided for the InfrastructureManager.")
+
         # First check if an OIDC token is included
         if "token" in im_auth[0]:
             InfrastructureManager.check_oidc_token(im_auth[0])
@@ -1282,6 +1428,8 @@ class InfrastructureManager:
 
         Return(int): the new infrastructure ID if successful.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
 
         # First check the auth data
         auth = InfrastructureManager.check_auth_data(auth)
@@ -1327,7 +1475,10 @@ class InfrastructureManager:
                         break
                 if all_failed:
                     # If all VMs has failed, destroy then inf and return the error
-                    inf.destroy(auth)
+                    try:
+                        inf.destroy(auth)
+                    except Exception as de:
+                        error_msg += "%s" % de
                     raise Exception(error_msg)
         except Exception as e:
             InfrastructureManager.logger.exception("Error Creating Inf ID " + str(inf.id))
@@ -1341,13 +1492,15 @@ class InfrastructureManager:
         return inf.id
 
     @staticmethod
-    def GetInfrastructureList(auth):
+    def GetInfrastructureList(auth, flt=None):
         """
         Return the infrastructure ids associated to IM tokens.
 
         Args:
 
         - auth(Authentication): parsed authentication tokens.
+        - flt(string): string to filter the list of returned infrastructures.
+                          A regex to be applied in the RADL or TOSCA of the infra.
 
         Return(list of int): list of infrastructure ids.
         """
@@ -1360,10 +1513,27 @@ class InfrastructureManager:
             InfrastructureManager.logger.error("No correct auth data has been specified.")
             raise InvaliddUserException()
 
-        return IM.InfrastructureList.InfrastructureList.get_inf_ids(auth)
+        inf_ids = IM.InfrastructureList.InfrastructureList.get_inf_ids(auth)
+        if flt:
+            res = []
+            for infid in inf_ids:
+                inf = InfrastructureManager.get_infrastructure(infid, auth)
+                radl = str(inf.get_radl())
+                tosca = ""
+                if "TOSCA" in inf.extra_info:
+                    tosca = inf.extra_info["TOSCA"].serialize()
+
+                if re.search(flt, radl) or re.search(flt, tosca):
+                    res.append(infid)
+        else:
+            res = inf_ids
+        return res
 
     @staticmethod
     def ExportInfrastructure(inf_id, delete, auth_data):
+        if delete and Config.BOOT_MODE == 1:
+            raise DisabledFunctionException()
+
         auth = Authentication(auth_data)
         auth = InfrastructureManager.check_auth_data(auth)
 
@@ -1378,6 +1548,9 @@ class InfrastructureManager:
 
     @staticmethod
     def ImportInfrastructure(str_inf, auth_data):
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = Authentication(auth_data)
         auth = InfrastructureManager.check_auth_data(auth)
 
@@ -1413,6 +1586,9 @@ class InfrastructureManager:
 
         Return: a str with url of the saved snapshot.
         """
+        if Config.BOOT_MODE in [1, 2]:
+            raise DisabledFunctionException()
+
         auth = InfrastructureManager.check_auth_data(auth)
         InfrastructureManager.logger.info("Creating a snapshot of VM id: %s Inf ID: %s" % (vm_id, inf_id))
 
