@@ -105,6 +105,8 @@ class Config:
     VERIFI_SSL = False
     SSH_REVERSE_TUNNELS = True
     ACTIVATE_XMLRPC = True
+    FORCE_OIDC_AUTH = False
+    BOOT_MODE = 0  # It can be 0-Normal, 1-ReadOnly, 2-ReadDelete
 
 
 config = ConfigParser()
