@@ -22,7 +22,10 @@ import socket
 import requests
 import random
 import uuid
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from IM.config import Config
 from .CloudConnector import CloudConnector
@@ -36,6 +39,8 @@ class DockerCloudConnector(CloudConnector):
     """
 
     type = "Docker"
+    DEFAULT_USER = 'root'
+    """ default user to SSH access the VM """
 
     _port_base_num = random.randint(35000, 40000)
     """ Base number to assign SSH port on Docker server host."""
@@ -78,7 +83,7 @@ class DockerCloudConnector(CloudConnector):
                         cert_file, key_file = cert
                         os.unlink(cert_file)
                         os.unlink(key_file)
-                    except:
+                    except Exception:
                         pass
 
         return resp
@@ -101,44 +106,23 @@ class DockerCloudConnector(CloudConnector):
 
         return (cert_file, key_file)
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+        if protocol == 'docker' and url[1]:
+            res_system = radl_system.clone()
+
+            res_system.addFeature(Feature("virtual_system_type", "=", "docker"), conflict="other", missing="other")
+
+            res_system.getFeature("cpu.count").operator = "="
+            res_system.getFeature("memory.size").operator = "="
+
+            res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
+            res_system.setValue('disk.0.os.credentials.password', self._root_password)
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
-
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-                if protocol == 'docker' and url[1]:
-                    res_system = radl_system.clone()
-
-                    res_system.addFeature(
-                        Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "virtual_system_type", "=", "docker"), conflict="other", missing="other")
-
-                    res_system.getFeature("cpu.count").operator = "="
-                    res_system.getFeature("memory.size").operator = "="
-
-                    res_system.setValue(
-                        'disk.0.os.credentials.username', 'root')
-                    res_system.setValue(
-                        'disk.0.os.credentials.password', self._root_password)
-
-                    res_system.addFeature(
-                        Feature("provider.type", "=", self.type), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.host", "=", self.cloud.server), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.port", "=", self.cloud.port), conflict="other", missing="other")
-
-                    res.append(res_system)
-
-            return res
+            return None
 
     def setIPs(self, vm, cont_info, auth_data):
         """
@@ -198,11 +182,7 @@ class DockerCloudConnector(CloudConnector):
         command += " ; "
         command += "mkdir /var/run/sshd"
         command += " ; "
-        command += " sed -i 's/#PermitRootLogin yes/PermitRootLogin yes/g' /etc/ssh/sshd_config "
-        command += " ; "
-        command += "sed -i 's/PermitRootLogin without-password/PermitRootLogin yes/g' /etc/ssh/sshd_config"
-        command += " ; "
-        command += "sed -i 's/PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config"
+        command += "sed -i '/PermitRootLogin/c\PermitRootLogin yes' /etc/ssh/sshd_config"
         command += " ; "
         command += "rm -f /etc/ssh/ssh_host_rsa_key*"
         command += " ; "
@@ -722,29 +702,23 @@ class DockerCloudConnector(CloudConnector):
             return (False, "Error connecting with Docker server")
 
     def stop(self, vm, auth_data):
-        try:
-            if self._is_swarm(auth_data):
-                return (False, "Not supported")
-
-            resp = self.create_request('POST', "/containers/" + vm.id + "/stop", auth_data)
-
-            if resp.status_code != 204:
-                return (False, "Error stopping the Container: " + resp.text)
-            else:
-                return (True, vm.id)
-        except Exception:
-            self.log_exception("Error connecting with Docker server")
-            return (False, "Error connecting with Docker server")
+        return self.cont_action(vm, 'stop', auth_data)
 
     def start(self, vm, auth_data):
+        return self.cont_action(vm, 'start', auth_data)
+
+    def reboot(self, vm, auth_data):
+        return self.cont_action(vm, 'restart', auth_data)
+
+    def cont_action(self, vm, action, auth_data):
         try:
             if self._is_swarm(auth_data):
                 return (False, "Not supported")
 
-            resp = self.create_request('POST', "/containers/" + vm.id + "/start", auth_data)
+            resp = self.create_request('POST', "/containers/" + vm.id + "/" + action, auth_data)
 
             if resp.status_code != 204:
-                return (False, "Error starting the Container: " + resp.text)
+                return (False, "Error in Container Action the Container: " + resp.text)
             else:
                 return (True, vm.id)
         except Exception:
