@@ -14,15 +14,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import time
 import uuid
 import os
+import time
 
 try:
-    from libcloud.compute.base import NodeSize
-    from libcloud.compute.types import NodeState, Provider
-    from libcloud.compute.providers import get_driver
-    from libcloud.common.google import ResourceNotFoundError
+    from libcloud.compute.base import NodeSize, NodeState
+    from libcloud.compute.types import Provider
+    from libcloud.compute.providers import get_driver as libcloud_get_driver
+    from libcloud.common.google import ResourceNotFoundError, ResourceExistsError
     from libcloud.dns.types import Provider as DNSProvider
     from libcloud.dns.types import RecordType
     from libcloud.dns.providers import get_driver as get_dns_driver
@@ -31,14 +31,17 @@ except Exception as ex:
     print("WARN: libcloud library not correctly installed. GCECloudConnector will not work!.")
     print(ex)
 
-from .CloudConnector import CloudConnector
-from IM.uriparse import uriparse
+from IM.connectors.LibCloud import LibCloudCloudConnector
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from radl.radl import Feature
 from IM.config import Config
 
 
-class GCECloudConnector(CloudConnector):
+class GCECloudConnector(LibCloudCloudConnector):
     """
     Cloud Launcher to GCE using LibCloud
     """
@@ -46,13 +49,15 @@ class GCECloudConnector(CloudConnector):
     type = "GCE"
     """str with the name of the provider."""
     DEFAULT_ZONE = "us-central1-a"
+    DEFAULT_USER = 'gceuser'
+    """ default user to SSH access the VM """
 
     def __init__(self, cloud_info, inf):
         self.auth = None
         self.datacenter = None
         self.driver = None
         self.dns_driver = None
-        CloudConnector.__init__(self, cloud_info, inf)
+        LibCloudCloudConnector.__init__(self, cloud_info, inf)
 
     def get_driver(self, auth_data, datacenter=None):
         """
@@ -77,7 +82,7 @@ class GCECloudConnector(CloudConnector):
             self.datacenter = datacenter
 
             if 'username' in auth and 'password' in auth and 'project' in auth:
-                cls = get_driver(Provider.GCE)
+                cls = libcloud_get_driver(Provider.GCE)
                 # Patch to solve some client problems with \\n
                 auth['password'] = auth['password'].replace('\\n', '\n')
                 lines = len(auth['password'].replace(" ", "").split())
@@ -85,8 +90,7 @@ class GCECloudConnector(CloudConnector):
                     raise Exception("The certificate provided to the GCE plugin has an incorrect format."
                                     " Check that it has more than one line.")
 
-                driver = cls(auth['username'], auth['password'],
-                             project=auth['project'], datacenter=datacenter)
+                driver = cls(auth['username'], auth['password'], project=auth['project'], datacenter=datacenter)
 
                 self.driver = driver
                 return driver
@@ -135,71 +139,46 @@ class GCECloudConnector(CloudConnector):
                 raise Exception(
                     "No correct auth data has been specified to GCE: username, password and project")
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+
+        if protocol == "gce":
+            driver = self.get_driver(auth_data)
+
+            res_system = radl_system.clone()
+            res_system.addFeature(Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
+
+            if res_system.getValue('availability_zone'):
+                region = res_system.getValue('availability_zone')
+            else:
+                region, _ = self.get_image_data(str_url)
+
+            instance_type = self.get_instance_type(driver.list_sizes(region), res_system)
+            if not instance_type:
+                return None
+
+            self.update_system_info_from_instance(res_system, instance_type)
+
+            username = res_system.getValue('disk.0.os.credentials.username')
+            if not username:
+                res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
+            return None
 
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-                if protocol == "gce":
-                    driver = self.get_driver(auth_data)
-
-                    res_system = radl_system.clone()
-                    res_system.addFeature(
-                        Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-
-                    if res_system.getValue('availability_zone'):
-                        region = res_system.getValue('availability_zone')
-                    else:
-                        region, _ = self.get_image_data(str_url)
-
-                    instance_type = self.get_instance_type(
-                        driver.list_sizes(region), res_system)
-
-                    if not instance_type:
-                        return []
-
-                    self.update_system_info_from_instance(
-                        res_system, instance_type)
-
-                    username = res_system.getValue(
-                        'disk.0.os.credentials.username')
-                    if not username:
-                        res_system.setValue(
-                            'disk.0.os.credentials.username', 'gceuser')
-                    res_system.addFeature(
-                        Feature("provider.type", "=", self.type), conflict="other", missing="other")
-
-                    res.append(res_system)
-
-            return res
-
-    def update_system_info_from_instance(self, system, instance_type):
+    @staticmethod
+    def update_system_info_from_instance(system, instance_type):
         """
         Update the features of the system with the information of the instance_type
         """
         if isinstance(instance_type, NodeSize):
-            system.addFeature(Feature(
-                "memory.size", "=", instance_type.ram, 'M'), conflict="other", missing="other")
-            if instance_type.disk:
-                system.addFeature(Feature(
-                    "disk.0.free_size", "=", instance_type.disk, 'G'), conflict="other", missing="other")
-            if instance_type.price:
-                system.addFeature(
-                    Feature("price", "=", instance_type.price), conflict="me", missing="other")
+            LibCloudCloudConnector.update_system_info_from_instance(system, instance_type)
             if 'guestCpus' in instance_type.extra:
                 system.addFeature(Feature("cpu.count", "=", instance_type.extra[
                                   'guestCpus']), conflict="other", missing="other")
 
-            system.addFeature(Feature(
-                "instance_type", "=", instance_type.name), conflict="other", missing="other")
-
     @staticmethod
     def set_net_provider_id(radl, net_name):
         """
@@ -227,9 +206,8 @@ class GCECloudConnector(CloudConnector):
             if net:
                 provider_id = net.getValue('provider_id')
                 if provider_id:
-                    break
+                    return provider_id
 
-        # TODO: check that the net exist in GCE
         return provider_id
 
     def get_instance_type(self, sizes, radl):
@@ -243,17 +221,7 @@ class GCECloudConnector(CloudConnector):
         """
         instance_type_name = radl.getValue('instance_type')
 
-        cpu = 1
-        cpu_op = ">="
-        if radl.getFeature('cpu.count'):
-            cpu = radl.getValue('cpu.count')
-            cpu_op = radl.getFeature('cpu.count').getLogOperator()
-
-        memory = 1
-        memory_op = ">="
-        if radl.getFeature('memory.size'):
-            memory = radl.getFeature('memory.size').getValue('M')
-            memory_op = radl.getFeature('memory.size').getLogOperator()
+        (cpu, cpu_op, memory, memory_op, _, _) = self.get_instance_selectors(radl)
 
         res = None
         for size in sizes:
@@ -262,12 +230,11 @@ class GCECloudConnector(CloudConnector):
             if size.price is None:
                 size.price = 9999
             if res is None or (size.price <= res.price or size.ram <= res.ram):
-                str_compare = ""
+                comparison = memory_op(size.ram, memory)
                 if 'guestCpus' in size.extra and size.extra['guestCpus']:
-                    str_compare = "size.extra['guestCpus'] " + cpu_op + " cpu and "
-                str_compare += "size.ram " + memory_op + " memory"
+                    comparison = comparison and cpu_op(size.extra['guestCpus'], cpu)
 
-                if eval(str_compare):
+                if comparison:
                     if not instance_type_name or size.name == instance_type_name:
                         res = size
 
@@ -283,7 +250,8 @@ class GCECloudConnector(CloudConnector):
 
         return res
 
-    def request_external_ip(self, radl):
+    @staticmethod
+    def request_external_ip(radl):
         """
         Check if the user has requested for a public ip
         """
@@ -329,7 +297,7 @@ class GCECloudConnector(CloudConnector):
            - path(str): URL of a VMI (some like this: gce://us-central1/debian-7 or gce://debian-7)
         Returns: a tuple (region, image_name) with the region and the AMI ID
         """
-        uri = uriparse(path)
+        uri = urlparse(path)
         if uri[2]:
             region = uri[1]
             image_name = uri[2][1:]
@@ -340,7 +308,8 @@ class GCECloudConnector(CloudConnector):
 
         return (region, image_name)
 
-    def get_default_net(self, driver):
+    @staticmethod
+    def get_default_net(driver):
         """
         Get the first net
         """
@@ -363,13 +332,32 @@ class GCECloudConnector(CloudConnector):
                 if net.isPublic():
                     public_net = net
 
+            # Create FW rules to allow all inside the VMs
+            if inf.id in net_name:
+                firewall_name = "%s-all" % net_name
+            else:
+                firewall_name = "im-%s-%s-all" % (inf.id, net_name)
+            allowed = [{'IPProtocol': 'udp', 'ports': '1-65535'},
+                       {'IPProtocol': 'tcp', 'ports': '1-65535'},
+                       {'IPProtocol': 'icmp'}]
+
+            try:
+                driver.ex_create_firewall(firewall_name, allowed, network=net_name, source_tags=['imid-%s' % inf.id])
+                self.log_info("Firewall %s successfully created." % firewall_name)
+            except ResourceExistsError:
+                self.log_debug("FW already exists. Ignore.")
+            except Exception as addex:
+                self.log_warn("Exception creating FW: " + str(addex))
+
             ports = {"tcp": ["22"]}
             if public_net:
+                if inf.id in net_name:
+                    firewall_name = "%s" % net_name
+                else:
+                    firewall_name = "im-%s-%s" % (inf.id, net_name)
+
                 outports = public_net.getOutPorts()
                 if outports:
-                    firewall_name = public_net.getValue("sg_name")
-                    if not firewall_name:
-                        firewall_name = "fw-im-%s" % net_name
                     for outport in outports:
                         if outport.get_protocol() not in ports:
                             ports[outport.get_protocol()] = []
@@ -379,32 +367,131 @@ class GCECloudConnector(CloudConnector):
                         elif outport.get_local_port() != 22:
                             ports[outport.get_protocol()].append(str(outport.get_remote_port()))
 
-                    allowed = [{'IPProtocol': 'tcp', 'ports': ports['tcp']}]
-                    if 'udp' in ports:
-                        allowed.append({'IPProtocol': 'udp', 'ports': ports['udp']})
+                allowed = [{'IPProtocol': 'tcp', 'ports': ports['tcp']}]
+                if 'udp' in ports:
+                    allowed.append({'IPProtocol': 'udp', 'ports': ports['udp']})
 
-                    firewall = None
+                try:
+                    driver.ex_create_firewall(firewall_name, allowed, network=net_name)
+                    self.log_info("Firewall %s successfully created." % firewall_name)
+                except ResourceExistsError:
+                    self.log_debug("FW already exists. Ignore.")
+                except Exception as addex:
+                    self.log_warn("Exception creating FW: " + str(addex))
+
+    def create_networks(self, driver, radl, inf):
+        """
+        Create GCE networks
+        """
+        try:
+            i = 0
+
+            while radl.systems[0].getValue("net_interface." + str(i) + ".connection"):
+                net_name = radl.systems[0].getValue("net_interface." + str(i) + ".connection")
+                i += 1
+                network = radl.get_network_by_id(net_name)
+                if network.getValue('create') == 'yes' and not network.isPublic():
+                    gce_net_name = "im-%s-%s" % (inf.id, net_name)
+
+                    # First check if the net already exists
+                    net = None
                     try:
-                        firewall = driver.ex_get_firewall(firewall_name)
-                    except ResourceNotFoundError:
-                        self.log_info("The firewall %s does not exist." % firewall_name)
-                    except:
-                        self.log_exception("Error trying to get FW %s." % firewall_name)
+                        net = driver.ex_get_network(gce_net_name)
+                    except Exception:
+                        self.log_debug("Net %s does not exist." % gce_net_name)
 
-                    if firewall:
-                        try:
-                            firewall.allowed = allowed
-                            firewall.update()
-                            self.log_info("Firewall %s existing. Rules updated." % firewall_name)
-                        except:
-                            self.log_exception("Error updating the firewall %s." % firewall_name)
-                        return
+                    if net:
+                        self.log_debug("Net %s already exist. Do not create it." % gce_net_name)
+                    else:
+                        net_cidr = network.getValue('cidr')
+                        if net_cidr:
+                            mode = "legacy"
+                            used_cidrs = [gce_net.cidr for gce_net in driver.ex_list_networks()]
+                            net_cidr = self.get_free_cidr(net_cidr, used_cidrs, inf)
+                        else:
+                            mode = "auto"
+                        self.log_info("Create net %s with cidr %s." % (gce_net_name, net_cidr))
+                        driver.ex_create_network(gce_net_name, net_cidr, "Net created by the IM", mode=mode)
+                        if net_cidr:
+                            network.setValue('cidr', net_cidr)
+                            # Set also the cidr in the inf RADL
+                            inf.radl.get_network_by_id(network.id).setValue('cidr', net_cidr)
 
-                    try:
-                        driver.ex_create_firewall(firewall_name, allowed, network=net_name)
-                        self.log_info("Firewall %s successfully created." % firewall_name)
-                    except Exception as addex:
-                        self.log_warn("Exception creating FW: " + str(addex))
+                    network.setValue('provider_id', gce_net_name)
+        except Exception as ext:
+            self.log_exception("Error creating networks.")
+            try:
+                self.delete_networks(driver, inf)
+            except Exception:
+                self.log_exception("Error deleting networks.")
+            raise Exception("Error creating networks: %s" % ext)
+
+        return True
+
+    def gen_disks_gce_struct(self, radl, driver, inf, image, location):
+        """
+        Return the required volumes (in the RADL) to be attached to the launched node
+
+        Arguments:
+           - radl(RADL): RADL document.
+           - driver: libCloud driver object
+           - inf: Infrastructure info
+           - image: image object
+           - location: disks location
+        Returns: A list of dicts as expected by the ex_disks_gce_struct
+        See: https://cloud.google.com/compute/docs/reference/rest/v1/instances
+        "disks" section
+        """
+        boot_disk_name = "bootd-%s" % uuid.uuid1()
+        boot_disk = {
+            'autoDelete': True,
+            'boot': True,
+            'type': 'PERSISTENT',
+            'mode': 'READ_WRITE',
+            'deviceName': boot_disk_name,
+            'initializeParams': {
+                'diskName': boot_disk_name,
+                'diskType': driver.ex_get_disktype('pd-standard', zone=location).extra['selfLink'],
+                'sourceImage': image.extra['selfLink']
+            }
+        }
+
+        res = [boot_disk]
+        cont = 1
+        while ((radl.systems[0].getValue("disk." + str(cont) + ".size") or
+                radl.systems[0].getValue("disk." + str(cont) + ".image.url")) and
+                radl.systems[0].getValue("disk." + str(cont) + ".device")):
+            disk_url = radl.systems[0].getValue("disk." + str(cont) + ".image.url")
+            disk_device = radl.systems[0].getValue("disk." + str(cont) + ".device")
+            disk_type = radl.systems[0].getValue("disk." + str(cont) + ".type")
+            if not disk_type:
+                disk_type = 'pd-standard'
+
+            disk = {'boot': False,
+                    'type': 'PERSISTENT',
+                    'mode': 'READ_WRITE',
+                    'deviceName': disk_device}
+
+            if disk_url:
+                # If the user has specified the volume name, try to get it
+                region, image_id = self.get_image_data(disk_url)
+                location = driver.ex_get_zone(region)
+                volume = driver.ex_get_volume(image_id, zone=location)
+                disk['source'] = volume.extra['selfLink']
+                disk['autoDelete'] = False
+            else:
+                disk_size = radl.systems[0].getFeature("disk." + str(cont) + ".size").getValue('G')
+                disk['autoDelete'] = True
+                disk['initializeParams'] = {'diskName': "im-%s-%s" % (inf.id, str(cont)),
+                                            'diskSizeGb': str(disk_size),
+                                            'diskType': driver.ex_get_disktype(disk_type,
+                                                                               zone=location).extra['selfLink']}
+
+            res.append(disk)
+
+            cont += 1
+
+        return res
 
     def launch(self, inf, radl, requested_radl, num_vm, auth_data):
         system = radl.systems[0]
@@ -439,14 +526,8 @@ class GCECloudConnector(CloudConnector):
                 'external_ip': None,
                 'location': region}
 
-        tags = {}
-        if system.getValue('instance_tags'):
-            keypairs = system.getValue('instance_tags').split(",")
-            for keypair in keypairs:
-                parts = keypair.split("=")
-                key = parts[0].strip()
-                value = parts[1].strip()
-                tags[key] = value
+        tags = self.get_instance_tags(system)
+        if tags:
             args['ex_metadata'] = tags
 
         # include the SSH_KEYS
@@ -456,7 +537,7 @@ class GCECloudConnector(CloudConnector):
 
         if not public or not private:
             # We must generate them
-            self.log_info("No keys. Generating key pair.")
+            self.log_debug("No keys. Generating key pair.")
             (public, private) = self.keygen()
             system.setValue('disk.0.os.credentials.private_key', private)
 
@@ -464,7 +545,6 @@ class GCECloudConnector(CloudConnector):
         if private and public:
             metadata = {"sshKeys": username + ":" + public}
             self.log_info("Setting ssh for user: " + username)
-            self.log_debug(metadata)
 
         startup_script = self.get_cloud_init_data(radl)
         if startup_script:
@@ -473,19 +553,24 @@ class GCECloudConnector(CloudConnector):
         if metadata:
             args['ex_metadata'] = metadata
 
-        net_provider_id = self.get_net_provider_id(radl)
-        if net_provider_id:
-            args['ex_network'] = net_provider_id
-            self.create_firewall(inf, net_provider_id, radl, driver)
-        else:
-            net_name = self.get_default_net(driver)
-            if net_name:
-                args['ex_network'] = net_name
-            else:
-                net_name = "default"
-            self.set_net_provider_id(radl, net_name)
-            self.create_firewall(inf, net_name, radl, driver)
+        # Create the info about disks
+        args['ex_disks_gce_struct'] = self.gen_disks_gce_struct(radl, driver, inf, image, region)
 
+        with inf._lock:
+            self.create_networks(driver, radl, inf)
+
+        net_provider_id = self.get_net_provider_id(radl)
+        if not net_provider_id:
+            net_provider_id = self.get_default_net(driver)
+            if not net_provider_id:
+                net_provider_id = "default"
+
+        args['ex_network'] = net_provider_id
+        self.create_firewall(inf, net_provider_id, radl, driver)
+
+        args['ex_can_ip_forward'] = self.can_ip_forward(radl)
+
+        node_tags = ["imid-%s" % inf.id]
         if self.request_external_ip(radl):
             args['external_ip'] = 'ephemeral'
             fixed_ip = self.request_fixed_ip(radl)
@@ -494,12 +579,16 @@ class GCECloudConnector(CloudConnector):
                     raise Exception("A fixed IP cannot be specified to a set of nodes (deploy is higher than 1)")
 
                 args['external_ip'] = driver.ex_create_address(name="im-" + fixed_ip, region=region, address=fixed_ip)
+        else:
+            node_tags = ["imid-%s-nopubip" % inf.id, "imid-%s" % inf.id]
 
+        args['ex_tags'] = node_tags
         res = []
         error_msg = "Error launching VM."
         if num_vm > 1:
             args['number'] = num_vm
             args['base_name'] = "%s-%s" % (name.lower().replace("_", "-"), str(uuid.uuid1()))
+            self.log_debug(args)
             try:
                 nodes = driver.ex_create_multiple_nodes(**args)
             except Exception as ex:
@@ -508,6 +597,7 @@ class GCECloudConnector(CloudConnector):
                 error_msg = str(ex)
         else:
             args['name'] = "%s-%s" % (name.lower().replace("_", "-"), str(uuid.uuid1()))
+            self.log_debug(args)
             try:
                 nodes = [driver.create_node(**args)]
             except Exception as ex:
@@ -534,11 +624,47 @@ class GCECloudConnector(CloudConnector):
             if args['external_ip'] != 'ephemeral':
                 try:
                     driver.ex_destroy_address(args['external_ip'])
-                except:
+                except Exception:
                     self.log_exception("Error deleting extenal IP.")
 
         return res
 
+    def delete_networks(self, driver, inf, timeout=120):
+        """
+        Delete created GCE networks
+        """
+        for gce_net in driver.ex_list_networks():
+            net_prefix = "im-%s-" % inf.id
+            if gce_net.name.startswith(net_prefix):
+                self.log_info("Deleting net %s." % gce_net.name)
+
+                cont = 0
+                deleted = False
+                while not deleted and cont < timeout:
+                    cont += 5
+                    try:
+                        gce_net.destroy()
+                        deleted = True
+                    except Exception as ex:
+                        self.log_warn("Error removing net: " + str(ex))
+
+                    if not deleted:
+                        time.sleep(5)
+
+        return True
+
+    def delete_routes(self, driver, inf):
+        """
+        Delete created GCE routes
+        """
+        for gce_route in driver.ex_list_routes():
+            name_prefix = "im-%s-" % inf.id
+            if gce_route.name.startswith(name_prefix):
+                self.log_info("Deleting route %s." % gce_route.name)
+                gce_route.destroy()
+
+        return True
+
     def finalize(self, vm, last, auth_data):
         try:
             if vm.id:
@@ -552,7 +678,6 @@ class GCECloudConnector(CloudConnector):
 
         if node:
             success = node.destroy()
-            self.delete_disks(node)
             self.del_dns_entries(vm, auth_data)
 
             if not success:
@@ -563,70 +688,21 @@ class GCECloudConnector(CloudConnector):
             self.log_warn("VM " + str(vm.id) + " not found.")
 
         if last:
-            self.delete_firewall(vm, auth_data)
+            driver = self.get_driver(auth_data)
+            self.delete_firewall(vm, driver)
+            self.delete_routes(driver, vm.inf)
+            self.delete_networks(driver, vm.inf)
 
         return (True, "")
 
-    def delete_firewall(self, vm, auth_data):
+    def delete_firewall(self, vm, driver):
         """
-        Delete the FW
+        Delete the FWs
         """
-        driver = self.get_driver(auth_data)
-        net_provider_id = self.get_net_provider_id(vm.info)
-        firewall_name = "fw-im-%s" % net_provider_id
-
-        firewall = None
-        try:
-            firewall = driver.ex_get_firewall(firewall_name)
-        except ResourceNotFoundError:
-            self.log_info("Firewall %s does not exist. Do not delete." % firewall_name)
-        except:
-            self.log_exception("Error trying to get FW %s." % firewall_name)
-
-        if firewall:
-            try:
-                firewall.destroy()
-                self.log_info("Firewall %s successfully deleted." % firewall_name)
-            except:
-                self.log_exception("Error trying to delete FW %s." % firewall_name)
-
-    def delete_disks(self, node):
-        """
-        Delete the disks of a node
-
-        Arguments:
-           - node(:py:class:`libcloud.compute.base.Node`): node object.
-        """
-        all_ok = True
-        for disk in node.extra['disks']:
-            try:
-                vol_name = os.path.basename(uriparse(disk['source'])[2])
-                volume = node.driver.ex_get_volume(vol_name)
-                # First try to detach the volume
-                if volume:
-                    success = volume.detach()
-                    if not success:
-                        self.log_error(
-                            "Error detaching the volume: " + vol_name)
-                    else:
-                        # wait a bit to detach the disk
-                        time.sleep(2)
-                    success = volume.destroy()
-                    if not success:
-                        self.log_error(
-                            "Error destroying the volume: " + vol_name)
-            except ResourceNotFoundError:
-                self.log_info("The volume: " + vol_name + " does not exists. Ignore it.")
-                success = True
-            except:
-                self.log_exception(
-                    "Error destroying the volume: " + vol_name + " from the node: " + node.id)
-                success = False
-
-            if not success:
-                all_ok = False
-
-        return all_ok
+        for gce_fw in driver.ex_list_firewalls():
+            if vm.inf.id in gce_fw.name:
+                self.log_info("Deleting FW %s." % gce_fw.name)
+                gce_fw.destroy()
 
     def get_node_with_id(self, node_id, auth_data):
         """
@@ -648,7 +724,85 @@ class GCECloudConnector(CloudConnector):
 
         return node
 
-    def get_node_location(self, node):
+    def can_ip_forward(self, radl):
+        """
+        Check if this node is specified as router
+        """
+        for network in radl.networks:
+            if network.getValue('router'):
+                router_info = network.getValue('router').split(",")
+                if len(router_info) != 2:
+                    self.log_error("Incorrect router format.")
+                    break
+
+                if radl.systems[0].name == router_info[1]:
+                    return True
+
+        return False
+
+    def addRouterInstance(self, vm, driver):
+        """
+        Set an instance as router
+        """
+        success = True
+        try:
+            i = 0
+            while vm.info.systems[0].getValue("net_interface." + str(i) + ".connection"):
+                net_name = vm.info.systems[0].getValue("net_interface." + str(i) + ".connection")
+                i += 1
+
+                network = vm.info.get_network_by_id(net_name)
+                if not network.getValue('provider_id'):
+                    gce_net = driver.ex_get_network('default')
+                else:
+                    gce_net = driver.ex_get_network(network.getValue('provider_id'))
+
+                if network.getValue('router'):
+                    router_info = network.getValue('router').split(",")
+                    if len(router_info) != 2:
+                        self.log_error("Incorrect router format.")
+                        success = False
+                        break
+
+                    system_router = router_info[1]
+                    router_cidr = router_info[0]
+
+                    vrouter = None
+                    for v in vm.inf.vm_list:
+                        if v.info.systems[0].name == system_router:
+                            vrouter = v.id
+                            break
+                    if not vrouter:
+                        self.log_error("No VRouter instance found with name %s" % system_router)
+                        success = False
+                        break
+
+                    vrouter_instance = driver.ex_get_node(vrouter)
+
+                    if vrouter_instance.state != NodeState.RUNNING:
+                        self.log_debug("VRouter instance %s is not running." % system_router)
+                        success = False
+                        break
+
+                    route_name = "im-%s-%s" % (vm.inf.id, net_name)
+                    self.log_info("Adding route %s to instance ID: %s." % (router_cidr, vrouter))
+                    try:
+                        driver.ex_create_route(route_name, router_cidr, priority=800, network=gce_net,
+                                               next_hop=vrouter_instance, tags=["imid-%s-nopubip" % vm.inf.id],
+                                               description="Route created by the IM")
+                    except ResourceExistsError:
+                        self.log_debug("Route already exists. Ignore.")
+
+                    # once set, delete it to not set it again
+                    network.delValue('router')
+        except Exception:
+            success = False
+            self.log_exception("Error adding Router Instance")
+
+        return success
+
+    @staticmethod
+    def get_node_location(node):
         """
         Get the location of a node
 
@@ -658,74 +812,6 @@ class GCECloudConnector(CloudConnector):
         """
         return node.extra['zone']
 
-    def wait_volume(self, volume, state='READY', timeout=60):
-        """
-        Wait a volume (with the state extra parameter) to be in certain state.
-
-        Arguments:
-           - volume(:py:class:`libcloud.compute.base.StorageVolume`): volume object or boolean.
-           - state(str): State to wait for (default value 'available').
-           - timeout(int): Max time to wait in seconds (default value 60).
-        """
-        if volume:
-            cont = 0
-            err_states = ["FAILED"]
-            while volume.extra['status'] != state and volume.extra['status'] not in err_states and cont < timeout:
-                cont += 2
-                time.sleep(2)
-                for vol in volume.driver.list_volumes():
-                    if vol.id == volume.id:
-                        volume = vol
-                        break
-            return volume.extra['status'] == state
-        else:
-            return False
-
-    def attach_volumes(self, vm, node):
-        """
-        Attach a the required volumes (in the RADL) to the launched node
-
-        Arguments:
-           - vm(:py:class:`IM.VirtualMachine`): VM information.
-           - node(:py:class:`libcloud.compute.base.Node`): node object.
-        """
-        try:
-            if node.state == NodeState.RUNNING and "volumes" not in vm.__dict__.keys():
-                vm.volumes = True
-                cont = 1
-                while (vm.info.systems[0].getValue("disk." + str(cont) + ".size") and
-                        vm.info.systems[0].getValue("disk." + str(cont) + ".device")):
-                    disk_size = vm.info.systems[0].getFeature(
-                        "disk." + str(cont) + ".size").getValue('G')
-                    disk_device = vm.info.systems[0].getValue(
-                        "disk." + str(cont) + ".device")
-                    self.log_info("Creating a %d GB volume for the disk %d" % (int(disk_size), cont))
-                    volume_name = "im-%s" % str(uuid.uuid1())
-
-                    location = self.get_node_location(node)
-                    volume = node.driver.create_volume(
-                        int(disk_size), volume_name, location=location)
-                    success = self.wait_volume(volume)
-                    if success:
-                        self.log_info("Attach the volume ID " + str(volume.id))
-                        try:
-                            volume.attach(node, disk_device)
-                        except:
-                            self.log_exception("Error attaching the volume ID " + str(
-                                volume.id) + " destroying it.")
-                            volume.destroy()
-                    else:
-                        self.log_error("Error waiting the volume ID " + str(
-                            volume.id) + " not attaching to the VM and destroying it.")
-                        volume.destroy()
-
-                    cont += 1
-            return True
-        except Exception:
-            self.log_exception(
-                "Error creating or attaching the volume to the node")
-            return False
-
     def updateVMInfo(self, vm, auth_data):
         driver = self.get_driver(auth_data)
 
@@ -740,29 +826,16 @@ class GCECloudConnector(CloudConnector):
             return (False, "Error getting VM info: %s. %s" % (vm.id, str(ex)))
 
         if node:
-            if node.state == NodeState.RUNNING or node.state == NodeState.REBOOTING:
-                res_state = VirtualMachine.RUNNING
-            elif node.state == NodeState.PENDING:
-                res_state = VirtualMachine.PENDING
-            elif node.state == NodeState.TERMINATED:
-                res_state = VirtualMachine.OFF
-            elif node.state == NodeState.STOPPED:
-                res_state = VirtualMachine.STOPPED
-            else:
-                res_state = VirtualMachine.UNKNOWN
-
-            vm.state = res_state
+            vm.state = self.VM_STATE_MAP.get(node.state, VirtualMachine.UNKNOWN)
 
             if 'zone' in node.extra:
-                vm.info.systems[0].setValue(
-                    'availability_zone', node.extra['zone'].name)
+                vm.info.systems[0].setValue('availability_zone', node.extra['zone'].name)
 
-            self.update_system_info_from_instance(
-                vm.info.systems[0], node.size)
+            self.update_system_info_from_instance(vm.info.systems[0], node.size)
 
             vm.setIps(node.public_ips, node.private_ips)
-            self.attach_volumes(vm, node)
             self.add_dns_entries(vm, auth_data)
+            self.addRouterInstance(vm, driver)
         else:
             vm.state = VirtualMachine.OFF
 
@@ -861,25 +934,15 @@ class GCECloudConnector(CloudConnector):
             return False
 
     def start(self, vm, auth_data):
-        driver = self.get_driver(auth_data)
-
-        try:
-            node = driver.ex_get_node(vm.id)
-        except ResourceNotFoundError:
-            return (False, "VM " + str(vm.id) + " does not exist.")
-        except Exception as ex:
-            self.log_exception("Error getting VM %s" % vm.id)
-            return (False, "Error getting VM %s: %s" % (vm.id, str(ex)))
-
-        try:
-            driver.ex_start_node(node)
-        except Exception as ex:
-            self.log_exception("Error starting VM %s" % vm.id)
-            return (False, "Error starting VM %s: %s" % (vm.id, str(ex)))
-
-        return (True, "")
+        return self.vm_action(vm, 'start', auth_data)
 
     def stop(self, vm, auth_data):
+        return self.vm_action(vm, 'stop', auth_data)
+
+    def reboot(self, vm, auth_data):
+        return self.vm_action(vm, 'reboot', auth_data)
+
+    def vm_action(self, vm, action, auth_data):
         driver = self.get_driver(auth_data)
 
         try:
@@ -891,10 +954,15 @@ class GCECloudConnector(CloudConnector):
             return (False, "Error getting VM %s: %s" % (vm.id, str(ex)))
 
         try:
-            driver.ex_stop_node(node)
+            if action == 'stop':
+                driver.ex_stop_node(node)
+            elif action == 'start':
+                driver.ex_start_node(node)
+            elif action == 'reboot':
+                driver.reboot_node(node)
         except Exception as ex:
-            self.log_exception("Error stopping VM %s" % vm.id)
-            return (False, "Error stopping VM %s: %s" % (vm.id, str(ex)))
+            self.log_exception("Error in VM action %s" % vm.id)
+            return (False, "Error in VM action %s: %s" % (vm.id, str(ex)))
 
         return (True, "")
 
