@@ -17,7 +17,10 @@
 import base64
 import json
 import requests
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from .CloudConnector import CloudConnector
 from radl.radl import Feature
@@ -64,7 +67,7 @@ class KubernetesCloudConnector(CloudConnector):
         """
         Generate the auth header needed to contact with the Kubernetes API server.
         """
-        url = uriparse(self.cloud.server)
+        url = urlparse(self.cloud.server)
         auths = auth_data.getAuthInfo(self.type, url[1])
         if not auths:
             self.log_error(
@@ -109,45 +112,22 @@ class KubernetesCloudConnector(CloudConnector):
         self.log_debug("Using %s API version." % version)
         return version
 
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+        if protocol == 'docker' and url[1]:
+            res_system = radl_system.clone()
+
+            res_system.addFeature(Feature("virtual_system_type", "=", "docker"), conflict="other", missing="other")
+            res_system.getFeature("cpu.count").operator = "="
+            res_system.getFeature("memory.size").operator = "="
+
+            res_system.setValue('disk.0.os.credentials.username', 'root')
+            res_system.setValue('disk.0.os.credentials.password', self._root_password)
+
+            return res_system
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
-
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-                if protocol == 'docker' and url[1]:
-                    res_system = radl_system.clone()
-
-                    res_system.addFeature(Feature(
-                        "virtual_system_type", "=", "docker"), conflict="other", missing="other")
-
-                    res_system.getFeature("cpu.count").operator = "="
-                    res_system.getFeature("memory.size").operator = "="
-
-                    res_system.setValue(
-                        'disk.0.os.credentials.username', 'root')
-                    res_system.setValue(
-                        'disk.0.os.credentials.password', self._root_password)
-
-                    res_system.addFeature(
-                        Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
-
-                    res_system.addFeature(
-                        Feature("provider.type", "=", self.type), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.host", "=", self.cloud.server), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.port", "=", self.cloud.port), conflict="other", missing="other")
-
-                    res.append(res_system)
-
-            return res
+            return None
 
     def _delete_volume_claim(self, namespace, vc_name, auth_data):
         try:
@@ -261,9 +241,7 @@ class KubernetesCloudConnector(CloudConnector):
         command += " ; "
         command += "mkdir /var/run/sshd"
         command += " ; "
-        command += "sed -i 's/PermitRootLogin without-password/PermitRootLogin yes/g' /etc/ssh/sshd_config"
-        command += " ; "
-        command += "sed -i 's/PermitRootLogin prohibit-password/PermitRootLogin yes/' /etc/ssh/sshd_config"
+        command += "sed -i '/PermitRootLogin/c\PermitRootLogin yes' /etc/ssh/sshd_config"
         command += " ; "
         command += "ssh-keygen -t rsa -f /etc/ssh/ssh_host_rsa_key -N ''"
         command += " ; "
@@ -369,7 +347,7 @@ class KubernetesCloudConnector(CloudConnector):
                     res.append((False, "Error creating the Container: " + resp.text))
                     try:
                         self._delete_volume_claims(pod_data, auth_data)
-                    except:
+                    except Exception:
                         self.log_exception("Error deleting volumes.")
                 else:
                     output = json.loads(resp.text)
@@ -427,7 +405,8 @@ class KubernetesCloudConnector(CloudConnector):
             self.log_error("Error getting info about the POD: code: %s, msg: %s" % (status, output))
             return (False, "Error getting info about the POD: code: %s, msg: %s" % (status, output))
 
-    def setIPs(self, vm, pod_info):
+    @staticmethod
+    def setIPs(vm, pod_info):
         """
         Adapt the RADL information of the VM to the real IPs assigned by the cloud provider
 
@@ -502,6 +481,9 @@ class KubernetesCloudConnector(CloudConnector):
     def start(self, vm, auth_data):
         return (False, "Not supported")
 
+    def reboot(self, vm, auth_data):
+        return (False, "Not supported")
+
     def alterVM(self, vm, radl, auth_data):
         # This function is correctly implemented
         # But kubernetes does not permit cpu to be updated yet
