@@ -22,15 +22,17 @@ try:
 except ImportError:
     from xmlrpc.client import ServerProxy
 
-import hashlib
+import os.path
 import time
-
 from distutils.version import LooseVersion
 from IM.xmlobject import XMLObject
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from .CloudConnector import CloudConnector
-from radl.radl import network, Feature
+from radl.radl import Feature
 from IM.config import ConfigOpenNebula
 from netaddr import IPNetwork, IPAddress
 from IM.config import Config
@@ -161,71 +163,62 @@ class OpenNebulaCloudConnector(CloudConnector):
 
     type = "OpenNebula"
     """str with the name of the provider."""
+    DEFAULT_USER = 'root'
+    """ default user to SSH access the VM """
 
     def __init__(self, cloud_info, inf):
         CloudConnector.__init__(self, cloud_info, inf)
-        self.server_url = "http://%s:%d/RPC2" % (
-            self.cloud.server, self.cloud.port)
-
-    def concreteSystem(self, radl_system, auth_data):
-        image_urls = radl_system.getValue("disk.0.image.url")
-        if not image_urls:
-            return [radl_system.clone()]
+        if self.cloud.path:
+            if self.cloud.port == -1:
+                if self.cloud.protocol == 'https':
+                    self.cloud.port = 443
+                elif self.cloud.protocol == 'http':
+                    self.cloud.port = 80
+                else:
+                    raise Exception("Invalid port/protocol specified for OpenNebula site: %s" % self.cloud.server)
+            self.server_url = "%s://%s:%d%s" % (self.cloud.protocol, self.cloud.server,
+                                                self.cloud.port, self.cloud.path)
         else:
-            if not isinstance(image_urls, list):
-                image_urls = [image_urls]
+            self.server_url = "http://%s:%d/RPC2" % (self.cloud.server, self.cloud.port)
 
-            res = []
-            for str_url in image_urls:
-                url = uriparse(str_url)
-                protocol = url[0]
-                src_host = url[1].split(':')[0]
-                # TODO: check the port
-                if (protocol == "one") and self.cloud.server == src_host:
-                    # Check the space in image and compare with disks.free_size
-                    if radl_system.getValue('disks.free_size'):
-                        disk_free = int(radl_system.getFeature(
-                            'disks.free_size').getValue('M'))
-                        # The VMRC specified the value in MB
-                        if radl_system.getValue("disk.0.size"):
-                            disk_size = int(radl_system.getValue("disk.0.size"))
-                        else:
-                            disk_size = 0
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+        src_host = url[1].split(':')[0]
 
-                        if disk_size < disk_free:
-                            # if the image do not have enough space, discard it
-                            return []
+        if (protocol == "one") and self.cloud.server == src_host:
+            # Check the space in image and compare with disks.free_size
+            if radl_system.getValue('disks.free_size'):
+                disk_free = int(radl_system.getFeature('disks.free_size').getValue('M'))
+                # The VMRC specified the value in MB
+                if radl_system.getValue("disk.0.size"):
+                    disk_size = int(radl_system.getValue("disk.0.size"))
+                else:
+                    disk_size = 0
 
-                    res_system = radl_system.clone()
+                if disk_size < disk_free:
+                    # if the image do not have enough space, discard it
+                    return []
 
-                    res_system.getFeature("cpu.count").operator = "="
-                    res_system.getFeature("memory.size").operator = "="
+            res_system = radl_system.clone()
 
-                    res_system.addFeature(
-                        Feature("disk.0.image.url", "=", str_url), conflict="other", missing="other")
+            res_system.getFeature("cpu.count").operator = "="
+            res_system.getFeature("memory.size").operator = "="
 
-                    res_system.addFeature(
-                        Feature("provider.type", "=", self.type), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.host", "=", self.cloud.server), conflict="other", missing="other")
-                    res_system.addFeature(Feature(
-                        "provider.port", "=", self.cloud.port), conflict="other", missing="other")
+            username = res_system.getValue('disk.0.os.credentials.username')
+            if not username:
+                res_system.setValue('disk.0.os.credentials.username', self.DEFAULT_USER)
 
-                    username = res_system.getValue('disk.0.os.credentials.username')
-                    if not username:
-                        res_system.setValue('disk.0.os.credentials.username', 'root')
+            return res_system
+        else:
+            return None
 
-                    res.append(res_system)
-
-            return res
-
-    def getSessionID(self, auth_data, hash_password=None):
+    def getSessionID(self, auth_data):
         """
         Get the ONE Session ID from the auth data
 
         Arguments:
            - auth_data(:py:class:`dict` of str objects): Authentication data to access cloud provider.
-           - hash_password(bool, optional): specifies if the password must be hashed
 
          Returns: str with the Session ID
         """
@@ -237,12 +230,6 @@ class OpenNebulaCloudConnector(CloudConnector):
 
         if 'username' in auth and 'password' in auth:
             passwd = auth['password']
-            if hash_password is None:
-                one_ver = self.getONEVersion(auth_data)
-                if one_ver == "2.0.0" or one_ver == "3.0.0":
-                    hash_password = True
-            if hash_password:
-                passwd = hashlib.sha1(passwd.strip().encode('utf-8')).hexdigest()
             return auth['username'] + ":" + passwd
         elif 'token' in auth:
             username, passwd = ONETTSClient.get_auth_from_tts(ConfigOpenNebula.TTS_URL,
@@ -255,7 +242,8 @@ class OpenNebulaCloudConnector(CloudConnector):
         else:
             raise Exception("No correct auth data has been specified to OpenNebula: username and password")
 
-    def setDisksFromTemplate(self, vm, template):
+    @staticmethod
+    def setDisksFromTemplate(vm, template):
         """
         Set the Disks of the VM from the info obtained in the ONE template object
 
@@ -266,14 +254,15 @@ class OpenNebulaCloudConnector(CloudConnector):
         for disk in template.DISK:
             vm.info.systems[0].setValue(
                 "disk." + str(disk.DISK_ID) + ".size", disk.SIZE, "M")
-            if disk.TARGET:
+            if disk.TARGET and not vm.info.systems[0].getValue("disk." + str(disk.DISK_ID) + ".device"):
                 vm.info.systems[0].setValue(
                     "disk." + str(disk.DISK_ID) + ".device", disk.TARGET)
-            if disk.FORMAT:
+            if disk.FORMAT and not vm.info.systems[0].getValue("disk." + str(disk.DISK_ID) + ".fstype"):
                 vm.info.systems[0].setValue(
                     "disk." + str(disk.DISK_ID) + ".fstype", disk.FORMAT)
 
-    def setIPsFromTemplate(self, vm, template):
+    @staticmethod
+    def setIPsFromTemplate(vm, template):
         """
         Set the IPs of the VM from the info obtained in the ONE template object
 
@@ -304,14 +293,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         if session_id is None:
             return (False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")
 
-        func_res = server.one.vm.info(session_id, int(vm.id))
-        if len(func_res) == 2:
-            (success, res_info) = func_res
-        elif len(func_res) == 3:
-            (success, res_info, _) = func_res
-        else:
-            return [(False, "Error in the one.vm.info return value")]
-
+        success, res_info = server.one.vm.info(session_id, int(vm.id))[0:2]
         if success:
             res_vm = VM(res_info)
 
@@ -369,7 +351,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         server = ServerProxy(self.server_url, allow_none=True)
         session_id = self.getSessionID(auth_data)
 
-        success, res_info, _ = server.one.secgrouppool.info(session_id, -1, -1, -1)
+        success, res_info = server.one.secgrouppool.info(session_id, -1, -1, -1)[0:2]
         if success:
             sg_pool = SECURITY_GROUP_POOL(res_info)
             for sg in sg_pool.SECURITY_GROUP:
@@ -418,7 +400,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                                                                            outport.get_port_init(),
                                                                            outport.get_port_end()))
                                 else:
-                                    if outport.get_remote_port() != 22:
+                                    if outport.get_remote_port() != 22 or not network.isPublic():
                                         sg_template += ("RULE = [ PROTOCOL = %s, RULE_TYPE = inbound, "
                                                         "RANGE = %d:%d ]\n" % (outport.get_protocol().upper(),
                                                                                outport.get_remote_port(),
@@ -427,7 +409,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                         if sg_template:
                             self.log_info("Creating security group: %s" % sg_name)
                             sg_template = ("NAME = %s\n" % sg_name) + sg_template
-                            success, sg_id, _ = server.one.secgroup.allocate(session_id, sg_template)
+                            success, sg_id = server.one.secgroup.allocate(session_id, sg_template)[0:2]
                             if not success:
                                 self.log_error("Error creating security group: %s" % sg_id)
 
@@ -460,13 +442,9 @@ class OpenNebulaCloudConnector(CloudConnector):
             inf.add_vm(vm)
             template = self.getONETemplate(vm.info, sgs, auth_data, vm)
 
-            func_res = server.one.vm.allocate(session_id, template)
-            if len(func_res) == 2:
-                (success, res_id) = func_res
-            elif len(func_res) == 3:
-                (success, res_id, _) = func_res
-            else:
-                return [(False, "Error in the one.vm.allocate return value")]
+            self.log_debug("ONE Template: %s" % template)
+
+            success, res_id = server.one.vm.allocate(session_id, template)[0:2]
 
             if success:
                 vm.id = str(res_id)
@@ -501,7 +479,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                 else:
                     try:
                         self.log_info("Deleting SG: %s" % sg_name)
-                        success, sg_id, _ = server.one.secgroup.delete(session_id, sg)
+                        success, sg_id = server.one.secgroup.delete(session_id, sg)[0:2]
                         if success:
                             self.log_info("Deleted.")
                             deleted = True
@@ -528,62 +506,35 @@ class OpenNebulaCloudConnector(CloudConnector):
             self.delete_snapshots(vm, auth_data)
 
         if vm.id:
-            func_res = server.one.vm.action(session_id, 'delete', int(vm.id))
-            if len(func_res) == 1:
-                success = True
-                err = vm.id
-            elif len(func_res) == 2:
-                (success, err) = func_res
-            elif len(func_res) == 3:
-                (success, err, _) = func_res
-            else:
-                return (False, "Error in the one.vm.action return value")
+            success, err = server.one.vm.action(session_id, 'delete', int(vm.id))[0:2]
         else:
             self.log_warn("No VM ID. Ignoring")
             err = ""
             success = True
 
         if last and success:
-            self.delete_security_groups(vm.inf, auth_data)
+            one_ver = self.getONEVersion(auth_data)
+            # Security Groups appears in version 4.12.0
+            if one_ver >= LooseVersion("4.12.0"):
+                self.delete_security_groups(vm.inf, auth_data)
 
         return (success, err)
 
     def stop(self, vm, auth_data):
-        server = ServerProxy(self.server_url, allow_none=True)
-        session_id = self.getSessionID(auth_data)
-        if session_id is None:
-            return (False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")
-        func_res = server.one.vm.action(session_id, 'suspend', int(vm.id))
-
-        if len(func_res) == 1:
-            success = True
-            err = vm.id
-        elif len(func_res) == 2:
-            (success, err) = func_res
-        elif len(func_res) == 3:
-            (success, err, _) = func_res
-        else:
-            return (False, "Error in the one.vm.action return value")
-
-        return (success, err)
+        return self.vm_action(vm, 'suspend', auth_data)
 
     def start(self, vm, auth_data):
+        return self.vm_action(vm, 'resume', auth_data)
+
+    def reboot(self, vm, auth_data):
+        return self.vm_action(vm, 'reboot-hard', auth_data)
+
+    def vm_action(self, vm, action, auth_data):
         server = ServerProxy(self.server_url, allow_none=True)
         session_id = self.getSessionID(auth_data)
         if session_id is None:
             return (False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")
-        func_res = server.one.vm.action(session_id, 'resume', int(vm.id))
-
-        if len(func_res) == 1:
-            success = True
-            err = vm.id
-        elif len(func_res) == 2:
-            (success, err) = func_res
-        elif len(func_res) == 3:
-            (success, err, _) = func_res
-        else:
-            return (False, "Error in the one.vm.action return value")
-
+        success, err = server.one.vm.action(session_id, action, int(vm.id))[0:2]
         return (success, err)
 
     def getONETemplate(self, radl, sgs, auth_data, vm):
@@ -608,38 +559,43 @@ class OpenNebulaCloudConnector(CloudConnector):
             name = system.getValue("disk.0.image.name")
         if not name:
             name = "userimage"
-        url = uriparse(system.getValue("disk.0.image.url"))
-        path = url[2]
+        url = urlparse(system.getValue("disk.0.image.url"))
+        path = os.path.basename(url[2])
 
-        if path[1:].isdigit():
-            disks = 'DISK = [ IMAGE_ID = "%s" ]\n' % path[1:]
+        if path.isdigit():
+            disks = 'DISK = [ IMAGE_ID = "%s" ]\n' % path
         else:
             if ConfigOpenNebula.IMAGE_UNAME:
                 # This only works if the user owns the image
-                disks = 'DISK = [ IMAGE = "%s" ]\n' % path[1:]
+                disks = 'DISK = [ IMAGE = "%s" ]\n' % path
             else:
                 disks = 'DISK = [ IMAGE = "%s", IMAGE_UNAME = "%s" ]\n' % (
-                    path[1:], ConfigOpenNebula.IMAGE_UNAME)
+                    path, ConfigOpenNebula.IMAGE_UNAME)
         cont = 1
         while system.getValue("disk." + str(cont) + ".image.url") or system.getValue("disk." + str(cont) + ".size"):
             disk_image = system.getValue("disk." + str(cont) + ".image.url")
+            disk_device = system.getValue("disk." + str(cont) + ".device")
             if disk_image:
-                disks += 'DISK = [ IMAGE_ID = "%s" ]\n' % uriparse(disk_image)[2][1:]
+                disks += 'DISK = [ IMAGE_ID = "%s"' % urlparse(disk_image)[2][1:]
             else:
                 disk_size = system.getFeature(
                     "disk." + str(cont) + ".size").getValue('M')
-                disk_device = system.getValue("disk." + str(cont) + ".device")
                 disk_fstype = system.getValue("disk." + str(cont) + ".fstype")
                 if not disk_fstype:
                     disk_fstype = 'ext3'
 
-                disks += ' DISK = [ TYPE = fs , FORMAT = %s, SIZE = %d,' % (disk_fstype, int(disk_size))
-                if disk_device:
-                    disks += 'TARGET = %s,' % disk_device
-                disks += 'SAVE = no ]\n'
+                disks += ' DISK = [ SAVE = no, TYPE = fs , FORMAT = %s, SIZE = %d' % (disk_fstype, int(disk_size))
+
+            if disk_device:
+                disks += ', TARGET = %s' % disk_device
+            disks += ' ]\n'
 
             cont += 1
 
+        sched = ""
+        if system.getValue('availability_zone'):
+            sched = 'SCHED_REQUIREMENTS = "CLUSTER_ID=\\"%s\\""' % system.getValue('availability_zone')
+
         res = '''
             NAME = %s
 
@@ -651,16 +607,14 @@ class OpenNebulaCloudConnector(CloudConnector):
             %s
 
             %s
-        ''' % (name, cpu, cpu, memory, arch, disks, ConfigOpenNebula.TEMPLATE_OTHER)
+
+            %s
+        ''' % (name, cpu, cpu, memory, arch, disks, sched, ConfigOpenNebula.TEMPLATE_OTHER)
 
         user_template = ""
-        if system.getValue('instance_tags'):
-            keypairs = system.getValue('instance_tags').split(",")
-            for keypair in keypairs:
-                parts = keypair.split("=")
-                key = parts[0].strip()
-                value = parts[1].strip()
-                user_template += '%s = "%s", ' % (key, value)
+        tags = self.get_instance_tags(system)
+        for key, value in tags.items():
+            user_template += '%s = "%s", ' % (key, value)
 
         if user_template:
             res += "\nUSER_TEMPLATE = [%s]\n" % user_template[:-2]
@@ -679,13 +633,12 @@ class OpenNebulaCloudConnector(CloudConnector):
             system.setValue('disk.0.os.credentials.private_key', private)
 
         if (private and public) or ConfigOpenNebula.TEMPLATE_CONTEXT or Config.SSH_REVERSE_TUNNELS:
-            res += 'CONTEXT = ['
+            context = ""
             if private and public:
-                res += 'SSH_PUBLIC_KEY = "%s"' % public
+                context += 'SSH_PUBLIC_KEY = "%s"' % public
 
-            if Config.SSH_REVERSE_TUNNELS:
-                if private and public:
-                    res += ", "
+            command = ""
+            if Config.SSH_REVERSE_TUNNELS and not vm.hasPublicNet():
                 inst_command = "apt update; apt install -y sshpass curl > /tmp/sshpass.out 2> /tmp/sshpass.err;"
                 inst_command += "yum install sshpass curl -y;"
                 inst_command += "zypper install -y sshpass curl;"
@@ -693,13 +646,22 @@ class OpenNebulaCloudConnector(CloudConnector):
                 command = "which sshpass && which curl || %s" % inst_command
 
                 command += vm.get_boot_curl_commands()[0]
-                res += 'START_SCRIPT = "%s"' % command.replace('"', '\\"')
+
+            if vm.getSSHPort() != 22:
+                command += "sed -i '/Port 22/c\\Port %d' /etc/ssh/sshd_config; " % vm.getSSHPort()
+                command += "service sshd restart; "
+
+            if command:
+                if context:
+                    context += ", "
+                context += 'START_SCRIPT = "%s"' % command.replace('"', '\\"')
 
             if ConfigOpenNebula.TEMPLATE_CONTEXT:
                 if private and public or Config.SSH_REVERSE_TUNNELS:
-                    res += ", "
-                res += ConfigOpenNebula.TEMPLATE_CONTEXT
-            res += ']'
+                    context += ", "
+                context += ConfigOpenNebula.TEMPLATE_CONTEXT
+            if context:
+                res += 'CONTEXT = [%s]' % context
 
         self.log_debug("Template: " + res)
 
@@ -719,8 +681,8 @@ class OpenNebulaCloudConnector(CloudConnector):
         version = "2.0.0"
         methods = server.system.listMethods()
         if "one.system.version" in methods:
-            session_id = self.getSessionID(auth_data, False)
-            (success, res_info, _) = server.one.system.version(session_id)
+            session_id = self.getSessionID(auth_data)
+            success, res_info = server.one.system.version(session_id)[0:2]
             if success:
                 version = res_info
             else:
@@ -734,7 +696,8 @@ class OpenNebulaCloudConnector(CloudConnector):
         self.log_debug("OpenNebula version: " + version)
         return version
 
-    def free_range(self, ar_range, total_leases):
+    @staticmethod
+    def free_range(ar_range, total_leases):
         """
         Check if there are at least one address free
 
@@ -752,7 +715,8 @@ class OpenNebulaCloudConnector(CloudConnector):
             return True
         return False
 
-    def free_address(self, addres_pool, used_leases):
+    @staticmethod
+    def free_address(addres_pool, used_leases):
         """
         Check if there are at least one address free
 
@@ -770,7 +734,8 @@ class OpenNebulaCloudConnector(CloudConnector):
             return True
         return False
 
-    def free_leases(self, leases):
+    @staticmethod
+    def free_leases(leases):
         """
         Check if there are at least one lease free
 
@@ -798,16 +763,7 @@ class OpenNebulaCloudConnector(CloudConnector):
         session_id = self.getSessionID(auth_data)
         if session_id is None:
             return None
-        func_res = server.one.vnpool.info(session_id, -2, -1, -1)
-
-        if len(func_res) == 2:
-            (success, info) = func_res
-        elif len(func_res) == 3:
-            (success, info, _) = func_res
-        else:
-            self.log_error("Error in the  one.vnpool.info return value")
-            return None
-
+        success, info = server.one.vnpool.info(session_id, -2, -1, -1)[0:2]
         if success:
             pool_info = VNET_POOL(info)
         else:
@@ -837,16 +793,7 @@ class OpenNebulaCloudConnector(CloudConnector):
             else:
                 self.log_warn(
                     "IP information is not in the VNET POOL. Use the vn.info")
-                info_res = server.one.vn.info(session_id, int(net.ID))
-
-                if len(info_res) == 2:
-                    (success, info) = info_res
-                elif len(func_res) == 3:
-                    (success, info, _) = info_res
-                else:
-                    self.log_warn(
-                        "Error in the one.vn.info return value. Ignoring network: " + net.NAME)
-                    continue
+                success, info = server.one.vn.info(session_id, int(net.ID))[0:2]
 
                 if not success:
                     self.log_warn(
@@ -883,7 +830,8 @@ class OpenNebulaCloudConnector(CloudConnector):
 
         return res
 
-    def map_radl_one_networks(self, radl_nets, one_nets):
+    @staticmethod
+    def map_radl_one_networks(radl_nets, one_nets):
         """
         Generate a mapping between the RADL networks and the ONE networks
 
@@ -1023,32 +971,14 @@ class OpenNebulaCloudConnector(CloudConnector):
         session_id = self.getSessionID(auth_data)
         if session_id is None:
             return (False, "Incorrect auth data, username and password must be specified for OpenNebula provider.")
-        func_res = server.one.vm.action(session_id, 'poweroff', int(vm.id))
-
-        if len(func_res) == 1:
-            success = True
-            err = vm.id
-        elif len(func_res) == 2:
-            (success, err) = func_res
-        elif len(func_res) == 3:
-            (success, err, _) = func_res
-        else:
-            return (False, "Error in the one.vm.action return value")
-
+        success, err = server.one.vm.action(session_id, 'poweroff', int(vm.id))[0:2]
         if not success:
             return (success, err)
 
         wait = 0
         powered_off = False
         while wait < timeout and not powered_off:
-            func_res = server.one.vm.info(session_id, int(vm.id))
-            if len(func_res) == 2:
-                (success, res_info) = func_res
-            elif len(func_res) == 3:
-                (success, res_info, _) = func_res
-            else:
-                return (False, "Error in the one.vm.info return value")
-
+            success, res_info = server.one.vm.info(session_id, int(vm.id))[0:2]
             res_vm = VM(res_info)
             powered_off = res_vm.STATE == 8
             if not powered_off:
@@ -1098,14 +1028,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                 ]
         ''' % (disk_fstype, disk_size, disk_device)
 
-        func_res = server.one.vm.attach(session_id, int(vm.id), disk_temp, False)
-        if len(func_res) == 2:
-            (success, res_info) = func_res
-        elif len(func_res) == 3:
-            (success, res_info, _) = func_res
-        else:
-            return (False, "Error in the one.vm.info return value")
-
+        success, res_info = server.one.vm.attach(session_id, int(vm.id), disk_temp, False)[0:2]
         if success:
             return (True, "")
         else:
@@ -1167,8 +1090,7 @@ class OpenNebulaCloudConnector(CloudConnector):
                 (success, info) = self.poweroff(vm, auth_data)
                 if not success:
                     return (success, info)
-                (success, info, _) = server.one.vm.resize(
-                    session_id, int(vm.id), new_temp, False)
+                success, info = server.one.vm.resize(session_id, int(vm.id), new_temp, False)[0:2]
                 self.start(vm, auth_data)
 
                 if success:
@@ -1196,15 +1118,11 @@ class OpenNebulaCloudConnector(CloudConnector):
         image_type = ""  # Use the default one
         one_ver = self.getONEVersion(auth_data)
         if one_ver.startswith("5."):
-            func_res = server.one.vm.disksaveas(session_id, int(vm.id), disk_num, image_name, image_type, -1)
+            success, res_info = server.one.vm.disksaveas(session_id, int(vm.id), disk_num,
+                                                         image_name, image_type, -1)[0:2]
         else:
-            func_res = server.one.vm.savedisk(session_id, int(vm.id), disk_num, image_name, image_type, True, False)
-        if len(func_res) == 2:
-            (success, res_info) = func_res
-        elif len(func_res) == 3:
-            (success, res_info, _) = func_res
-        else:
-            return (False, "Error in the one.vm.savedisk return value")
+            success, res_info = server.one.vm.savedisk(session_id, int(vm.id), disk_num,
+                                                       image_name, image_type, True, False)[0:2]
 
         if success:
             new_url = "one://%s/%d" % (self.cloud.server, res_info)
@@ -1215,8 +1133,8 @@ class OpenNebulaCloudConnector(CloudConnector):
                 return (True, new_url)
             else:
                 try:
-                    (success, res_info, _) = server.one.image.delete(session_id, res_info)
-                except:
+                    success, res_info, = server.one.image.delete(session_id, res_info)[0:2]
+                except Exception:
                     self.logger.error("Error deleting image: %s" % res_info)
                 return (False, "Error waiting image to be ready: %s" % msg)
         else:
@@ -1235,14 +1153,7 @@ class OpenNebulaCloudConnector(CloudConnector):
             wait += 5
             time.sleep(5)
 
-            func_res = server.one.image.info(session_id, image_id)
-            if len(func_res) == 2:
-                (success, res_info) = func_res
-            elif len(func_res) == 3:
-                (success, res_info, _) = func_res
-            else:
-                return (False, "Error in the one.image.info return value")
-
+            success, res_info = server.one.image.info(session_id, image_id)[0:2]
             if success:
                 image_info = IMAGE(res_info)
                 state = image_info.STATE
@@ -1273,36 +1184,21 @@ class OpenNebulaCloudConnector(CloudConnector):
         if not success:
             self.logger.warn("Error waiting image to be READY: " + msg)
 
-        func_res = server.one.image.delete(session_id, image_id)
-        if len(func_res) == 2:
-            (success, res_info) = func_res
-        elif len(func_res) == 3:
-            (success, res_info, _) = func_res
-        else:
-            return (False, "Error in the one.image.delete return value")
-
+        success, res_info = server.one.image.delete(session_id, image_id)[0:2]
         if success:
             return (True, "")
         else:
             return (False, res_info)
 
     def get_image_id(self, image_url, session_id):
-        url = uriparse(image_url)
+        url = urlparse(image_url)
         image_id = url[2][1:]
         if image_id.isdigit():
             return int(image_id)
         else:
             # We have to find the ID of the image name
             server = ServerProxy(self.server_url, allow_none=True)
-            func_res = server.one.imagepool.info(session_id, -2, -1, -1)
-            if len(func_res) == 2:
-                (success, res_info) = func_res
-            elif len(func_res) == 3:
-                (success, res_info, _) = func_res
-            else:
-                self.logger.error("Error in the one.imagepool.info return value")
-                return None
-
+            success, res_info = server.one.imagepool.info(session_id, -2, -1, -1)[0:2]
             if success:
                 pool_info = IMAGE_POOL(res_info)
             else:
