@@ -24,8 +24,10 @@ except Exception as ex:
     print("WARN: VMWare pyVmomi library not correctly installed. vSphereCloudConnector will not work!.")
     print(ex)
 
-from radl.radl import Feature
-from IM.uriparse import uriparse
+try:
+    from urlparse import urlparse
+except ImportError:
+    from urllib.parse import urlparse
 from IM.VirtualMachine import VirtualMachine
 from IM.config import Config
 from .CloudConnector import CloudConnector
@@ -137,41 +139,33 @@ class vSphereCloudConnector(CloudConnector):
                 self.log_debug(auth)
                 raise Exception("No correct auth data has been specified to vSpere: username, password")
 
-    def concreteSystem(self, radl_system, auth_data):
-        if radl_system.getValue("disk.0.image.url"):
-            url = uriparse(radl_system.getValue("disk.0.image.url"))
-            protocol = url[0]
-            src_host = url[1].split(':')[0]
-            if protocol == "vsp" and self.cloud.server == src_host:
-                # Check the space in image and compare with disks.free_size
-                if radl_system.getValue('disks.free_size'):
-                    disk_free = int(radl_system.getFeature('disks.free_size').getValue('M'))
-                    # The VMRC specified the value in MB
-                    disk_size = int(radl_system.getValue("disk.0.size"))
+    def concrete_system(self, radl_system, str_url, auth_data):
+        url = urlparse(str_url)
+        protocol = url[0]
+        src_host = url[1].split(':')[0]
 
-                    if disk_size < disk_free:
-                        # if the image do not have enough space, discard it
-                        return []
+        if protocol == "vsp" and self.cloud.server == src_host:
+            # Check the space in image and compare with disks.free_size
+            if radl_system.getValue('disks.free_size'):
+                disk_free = int(radl_system.getFeature('disks.free_size').getValue('M'))
+                # The VMRC specified the value in MB
+                disk_size = int(radl_system.getValue("disk.0.size"))
 
-                res_system = radl_system.clone()
+                if disk_size < disk_free:
+                    # if the image do not have enough space, discard it
+                    return None
 
-                res_system.getFeature("cpu.count").operator = "="
-                res_system.getFeature("memory.size").operator = "="
+            res_system = radl_system.clone()
 
-                res_system.addFeature(Feature("provider.type", "=", self.type),
-                                      conflict="other", missing="other")
-                res_system.addFeature(Feature("provider.host", "=", self.cloud.server),
-                                      conflict="other", missing="other")
-                res_system.addFeature(Feature("provider.port", "=", self.cloud.port),
-                                      conflict="other", missing="other")
+            res_system.getFeature("cpu.count").operator = "="
+            res_system.getFeature("memory.size").operator = "="
 
-                return [res_system]
-            else:
-                return []
+            return res_system
         else:
-            return [radl_system.clone()]
+            return None
 
-    def gen_nic(self, num, network):
+    @staticmethod
+    def gen_nic(num, network):
         """
         Get a nic for the specified network
         """
@@ -310,7 +304,8 @@ class vSphereCloudConnector(CloudConnector):
         return task
 
     # The path must be: vsp://template_name
-    def get_template_name(self, path):
+    @staticmethod
+    def get_template_name(path):
         """
         Get the region and the image name from an URL of a VMI
 
@@ -318,7 +313,7 @@ class vSphereCloudConnector(CloudConnector):
            - path(str): URL of a VMI (some like this: vsp://template_name)
         Returns: a str with the template name
         """
-        uri = uriparse(path)
+        uri = urlparse(path)
         return uri[2][1:]
 
     @staticmethod
@@ -332,7 +327,8 @@ class vSphereCloudConnector(CloudConnector):
             obj[c.name] = c
         return obj
 
-    def map_radl_vsphere_networks(self, radl_nets, vsphere_nets):
+    @staticmethod
+    def map_radl_vsphere_networks(radl_nets, vsphere_nets):
         """
         Generate a mapping between the RADL networks and the ONE networks
 
@@ -479,7 +475,7 @@ class vSphereCloudConnector(CloudConnector):
                 task = node.PowerOff()
                 try:
                     self.wait_for_tasks(connection, [task])
-                except:
+                except Exception:
                     self.log_exception("Error powering off VM " + str(vm.id))
 
             task = node.Destroy()
@@ -494,7 +490,8 @@ class vSphereCloudConnector(CloudConnector):
             self.log_warn("VM " + str(vm.id) + " not found.")
         return (True, "")
 
-    def setIps(self, vm, node):
+    @staticmethod
+    def setIps(vm, node):
         """
         Set the IPs of the VM from the info obtained from vSphere
         """
@@ -529,41 +526,37 @@ class vSphereCloudConnector(CloudConnector):
         return (True, vm)
 
     def start(self, vm, auth_data):
-        connection = self.get_connection(auth_data)
-        node = self.get_vm_by_name(connection.RetrieveContent(), vm.id)
-
-        if node:
-            if format(node.runtime.powerState) == "suspended":
-                task = node.PowerOn()
-                try:
-                    self.wait_for_tasks(connection, [task])
-                except Exception as ex:
-                    self.log_exception("Error starting VM " + str(vm.id))
-                    return (False, "Error starting the VM: " + str(ex))
-            else:
-                return (False, "Error starting the VM. The VM is not suspended.")
-
-            self.log_debug("VM " + str(vm.id) + " successfully started")
-        else:
-            self.log_warn("VM " + str(vm.id) + " not found.")
-        return (True, "")
+        return self.vm_action(vm, 'start', auth_data)
 
     def stop(self, vm, auth_data):
+        return self.vm_action(vm, 'stop', auth_data)
+
+    def reboot(self, vm, auth_data):
+        return self.vm_action(vm, 'reboot', auth_data)
+
+    def vm_action(self, vm, action, auth_data):
         connection = self.get_connection(auth_data)
         node = self.get_vm_by_name(connection.RetrieveContent(), vm.id)
 
         if node:
-            if format(node.runtime.powerState) == "poweredOn":
+            if action == 'stop':
+                if format(node.runtime.powerState) != "poweredOn":
+                    return (False, "Error stopping the VM. The VM is not running.")
                 task = node.Suspend()
-                try:
-                    self.wait_for_tasks(connection, [task])
-                except Exception as ex:
-                    self.log_exception("Error stopping VM " + str(vm.id))
-                    return (False, "Error stopping the VM: " + str(ex))
-            else:
-                return (False, "Error stopping the VM. The VM is not running.")
+            elif action == 'start':
+                if format(node.runtime.powerState) != "suspended":
+                    return (False, "Error starting the VM. The VM is not suspended.")
+                task = node.PowerOn()
+            elif action == 'reboot':
+                task = node.Reset()
 
-            self.log_debug("VM " + str(vm.id) + " successfully stopping")
+            try:
+                self.wait_for_tasks(connection, [task])
+            except Exception as ex:
+                self.log_exception("Error in VM action " + str(vm.id))
+                return (False, "Error in VM action: " + str(ex))
+
+            self.log_debug("VM " + str(vm.id) + " successfully " + action)
         else:
             self.log_warn("VM " + str(vm.id) + " not found.")
         return (True, "")
@@ -642,5 +635,5 @@ class vSphereCloudConnector(CloudConnector):
                     self.log_info("Creating a %d GB volume for the disk %d" % (int(disk_size), cont))
                     self.add_disk(node, conn, disk_size)
                     cont += 1
-        except:
+        except Exception:
             self.log_exception("Error creating or attaching the volume to the instance")
